/* $Id: translator.c,v 1.2 2004/01/19 22:31:30 jgordon Exp $ */

/* 
 * $Log: translator.c,v $
 * Revision 1.2  2004/01/19 22:31:30  jgordon
 * merge from GATEWAY_PRODUCT_1_BRANCH (MAINLINE_MP_1) to mainline
 *
 * Revision 1.1.2.1  2004/01/06 01:52:30  alan
 * Threaded interpreter engine
 *
 * Revision 1.1.2.1.2.3.2.2  1999/11/21 16:51:19  helmut
 * minor changes
 *
 * Revision 1.1.2.1.2.5  1999/11/21 16:44:33  helmut
 * alloc BYTECODE in rather than DISPATCHTABLE.
 *
 * Revision 1.1.2.1.2.4  1999/11/21 16:39:29  helmut
 * minor changes
 *
 * Revision 1.1.2.1.2.3.2.1  1999/11/15 23:38:17  helmut
 * use Instr union in place of casts to increase portability.
 *
 * Revision 1.1.2.1.2.3  1999/11/14 11:59:33  helmut
 * use GC_ALLOC_BYTECODE.
 *
 * Revision 1.1.2.1.2.2  1999/11/11 18:50:41  helmut
 * *** empty log message ***
 *
 * Revision 1.1.2.1.2.1  1999/11/10 11:49:28  helmut
 * *** empty log message ***
 *
 */


#include "debug.h"
#define	CDBG(s) 	DBG(INT_VMCALL, s)
#define	RDBG(s) 	DBG(INT_RETURN, s)
#define	NDBG(s) 	DBG(INT_NATIVE, s)
#define	IDBG(s)		DBG(INT_INSTR, s)
#define	CHDBG(s)	DBG(INT_CHECKS, s)

#include "config.h"
#include "config-std.h"
#include "config-math.h"
#include "config-mem.h"
#include "config-setjmp.h"
#include "classMethod.h"
#include "gtypes.h"
#include "bytecode.h"
#include "slots.h"
#include "icode.h"
#include "access.h"
#include "object.h"
#include "constants.h"
#include "gc.h"
#include "machine.h"
#include "lookup.h"
#include "code-analyse.h"
#include "soft.h"
#include "exception.h"
#include "external.h"
#include "baseClasses.h"
#include "thread.h"
#include "jthread.h"
#include "locks.h"
#include "checks.h"
#include "errors.h"
#include "md.h"


#include "translator.h"
#include "engine.h"

static char* label2string[256+N_CUSTOM_INSTRUCTIONS+1] = {
# define instr(name,opcode,label,len,syn,body) #label,
# define undef(name,opcode,label,len,syn,body) #label,
# define custm(name,opcode,label,len,syn,body) #label,
# include "threaded_int.def"
# undef instr
# undef undef
# undef custm
  "dummy"
}; 

static char* mnemonic2string[256+1] = {
# define instr(name,opcode,label,len,syn,body) #name,
# define undef(name,opcode,label,len,syn,body) #name,
# define custm(name,opcode,label,len,syn,body) 
# include "threaded_int.def"
# undef instr
# undef undef
# undef custm
  "dummy"
}; 


char *opcode_2_string(unsigned char opcode) {
	return mnemonic2string[opcode];
}

Instr opcode_2_label(unsigned char opcode) {
	static Instr *b2i = NULL;
	if(b2i == NULL) b2i = get_labels();
	return b2i[opcode];
}

char *label_2_string(Instr label) {
	return label2string[label_2_opcode(label)]; 
}
unsigned label_2_opcode(Instr label) {
	unsigned i;
	static Instr *b2i = NULL;
	if(b2i == NULL) b2i = get_labels();
	for(i=0;i<=255+N_CUSTOM_INSTRUCTIONS;i++) 
	    if (label.p == b2i[i].p) return i;
	fprintf(stderr,"invalid label %p\n", label.p);
	abort();
}
Instr string_2_label(char *label_string) {
	unsigned  i;
	static Instr *b2i = NULL;
	if(b2i == NULL) b2i = get_labels();
	for(i=0;i<=255+N_CUSTOM_INSTRUCTIONS;i++) 
	    if (strcmp(label_2_string(b2i[i]), label_string) == 0) 
		return b2i[i];
	fprintf(stderr,"invalid label-string \"%s\"\n", label_string);
	abort();
}
unsigned char string_2_opcode(char *opcode_string) {
	unsigned char i;
	for(i=0;i<=255;i++) 
	    if (strcmp(opcode_2_string(i), opcode_string) == 0) return i;
	fprintf(stderr,"invalid mnemonic \"%s\"\n", opcode_string);
	abort();
}

unsigned ip_2_pc(Instr* ip, methods *mp) {
	return ip-mp->icode;
}

Instr* pc_2_ip(unsigned pc, methods *mp) {
	return mp->icode+pc;
}

void translate_to_icode(methods *meth) {
	Instr *icode = meth->icode;  
	unsigned char *bcode = meth->c.bcode.code;
	Instr *b2i;
	unsigned i=0;
	unsigned return_modifier = IRETURN;
	static void *tcode[256+1] = {
#	  define instr(name,opcode,label,length,syntax,body) &&##syntax,
#	  define undef(name,opcode,label,length,syntax,body) &&undef,
#	  define custm(name,opcode,label,length,syntax,body) 
#         include "threaded_int.def"       			
#	  undef instr
#	  undef undef
#	  undef custm
	  &&undef
	};
	void assign_uint8(unsigned to, unsigned from) {
		icode[to].u = (uint8)(bcode[from]);
		IDBG(dprintf("%#x (uint8)=> %#lx\n", 
			    bcode[from], icode[to].p));
	}
	void assign_int8(unsigned to, unsigned from) {
		icode[to].i = (int8)(bcode[from]);
		IDBG(dprintf("%#x (int8)=> %#lx\n", 
			    bcode[from], icode[to].p));
	}
	void assign_uint16(unsigned to, unsigned from) {
		icode[to].u = (uint16)(bcode[from]<<8|bcode[from+1]);
		IDBG(dprintf("%#x %#x (uint16)=> %#lx\n", 
			     bcode[from], bcode[from+1], icode[to].p));
	}
	void assign_int16(unsigned to, unsigned from) {
		icode[to].i = (int16)(bcode[from]<<8|bcode[from+1]);
		IDBG(dprintf("%#x %#x (int16)=> %#lx\n", 
			     bcode[from], bcode[from+1], icode[to].p));
	}
	void assign_uint32(unsigned to, unsigned from) {
		icode[to].u = 
		  (uint32)(bcode[from  ]<<24 | bcode[from+1]<<16 | 
			   bcode[from+2]<< 8 | bcode[from+3]); 
		IDBG(dprintf("%#x %#x %#x %#x (uint32)=> %#lx\n", 
			     bcode[from], bcode[from+1],
			     bcode[from+2], bcode[from+3], icode[to].p));
	}
	void assign_int32(unsigned to, unsigned from) {
		icode[to].i = 
		    (int32)(bcode[from  ]<<24 | bcode[from+1]<<16 | 
			    bcode[from+2]<< 8 | bcode[from+3]); 
		IDBG(dprintf("%#x %#x %#x %#x (int32)=> %#lx\n", 
			    bcode[from], bcode[from+1],
			    bcode[from+2], bcode[from+3], icode[to].p));
	}
	IDBG(dprintf("translate...: %s.%s%s.\n", meth->class->name->data, 
		     meth->name->data, METHOD_SIGD(meth)));
	assert(meth->c.bcode.codelen>0);
	icode = gc_malloc(meth->c.bcode.codelen*sizeof *icode,
			  GC_ALLOC_BYTECODE);
	assert(icode);
	b2i = get_labels();
	for(i=0;i < meth->c.bcode.codelen;) {
		icode[i]=b2i[bcode[i]];
		IDBG(dprintf("%s(%u) => %s(%p)\n", 
			     opcode_2_string(bcode[i]), bcode[i],
			     label_2_string(icode[i]), icode[i]));
		goto *tcode[bcode[i]]; 
	op:  
		i++; continue;
	op_uint8:  
		assign_uint8(i+1,i+1);	i += 2; continue;
	op_int8:   
		assign_int8(i+1,i+1);	i += 2; continue;
	op_uint16: 
		assign_uint16(i+1,i+1);	i += 3; continue;
	op_int16:  
		assign_int16(i+1,i+1);	i += 3; continue;
	op_int32:  
		assign_int32(i+1,i+1);	i += 5; continue;
	op_uint8_int8: 
		assign_uint8(i+1,i+1); assign_int8(i+2,i+2);   i+=3; continue; 
	op_uint16_uint8:
		assign_uint16(i+1,i+1);assign_uint8(i+2,i+3); i+=4; continue;
	special_return:
		icode[i]=b2i[bcode[i]-IRETURN+return_modifier]; i++; continue;
	special_wide:
		icode[i]=b2i[bcode[i+1]]; 
		IDBG(dprintf("%s(%u) (wide)=> %s(%p)\n", 
			     opcode_2_string(bcode[i+1]), bcode[i+1],
			     label_2_string(icode[i]), icode[i]));
		switch(bcode[i+1]) {
		case IINC:
		    assign_uint16(i+1,i+2);
		    assign_int16(i+2,i+4);
		    i += 6; continue; 
		case ILOAD: case FLOAD: case  ALOAD: 
		case LLOAD: case DLOAD: case ISTORE: 
		case FSTORE: case ASTORE: case LSTORE: 
		case DSTORE: case RET:
		    assign_uint16(i+1,i+2);
		    i += 4; continue; 
		default:
		    fprintf(stderr,"invalid wide opcode %s(%i)\n",
			    opcode_2_string(bcode[i+1]), bcode[i+1]);
		    assert(0);
		}
	special_lookupswitch: 
		{
				/* padding is in [0..3]; padding
				   should be 3 if i=0 */
			unsigned padding = 3-i%4;
			unsigned base = i+1+padding;
			int32 npairs, j;
			assign_int32(i+1,base+0*4); /* default */
			assign_int32(i+2,base+1*4); /* npairs */
			npairs = icode[i+2].i; 
			for(j=0;j<npairs;j++) {
				assign_int32(i+3+j*2, base+2*4+j*8);
				assign_int32(i+4+j*2, base+3*4+j*8);
			}
			i += 1+padding+8+(npairs*2)*4; continue;
		}
	special_tableswitch: 
		{
				/* padding is in [0..3]; padding
                                   should be 3 if i=0 */
			unsigned padding = 3-i%4;
			unsigned base = i+1+padding;
			int32 low, high, j;
			assign_int32(i+1,base+0*4); /* default */
			assign_int32(i+2,base+1*4); /* low */
			assign_int32(i+3,base+2*4); /* high */
			low = icode[i+2].i;
			high = icode[i+3].i;
			for(j=0;j<high-low+1;j++)
			    assign_int32(i+4+j,base+3*4+j*4);
			i += 1+padding+12+4*(high-low+1); continue;
		}
	special_invokeinterface:
		assign_uint16(i+1,i+1);
		assign_int8(i+3,i+3);
		/* bcode[i+4] is empty */
		i += 5; continue;
	special_breakpoint:
		fprintf(stderr,"Don't know what do at opcode `breakpoint'.\n");
		abort();
	undef:
		fprintf(stderr,"undefined opcode\n");
		assert(0);
	}
	meth->icode = icode;
	IDBG(dprintf("translate...done\n"));
} 



