#include <jni.h>
#include "i802.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <unistd.h>

#include <fcntl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <signal.h>
#include <errno.h>

/* getHardwareAddress( name ) */
JNIEXPORT 
jstring JNICALL
 Java_com_instant802_cldc_linux_NetInterface_nativeGetHardwareAddress(JNIEnv *env, jobject obj, jstring name)
{
	int sd;
	struct ifreq *req;
	jsize interfaceNameLength;
	char *interfaceName;
	char *formatted;
	jboolean isnull;
	jstring hardwareAddress;
	
	/* set the return value to null in case we don't get the result */
	hardwareAddress = NULL;
	
	/* copy the name into a c string */
	interfaceName =  (char *) ((*env)->GetStringUTFChars(env, name, NULL));	

	/* open the socket on which to call the ioctl */
	sd = socket(AF_INET, SOCK_DGRAM, 0);

	if( sd > 0 ) {

		/* allocate a buffer for the request */
		req =malloc( sizeof( struct ifreq ) );

		/* clear the memory used for the request */
		memset( req, 0, sizeof( struct ifreq ));

		/* copy the name into the request */
		strncpy( req->ifr_name, interfaceName,  16 );

		/* set the address family to 'internet' */
		req->ifr_addr.sa_family = AF_INET;

		/* perfom the ioctl */
		if( ioctl(sd, SIOCGIFHWADDR, req) >=0 ) {
			formatted = malloc(32);
			sprintf(formatted, "%02x:%02x:%02x:%02x:%02x:%02x",
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[0],
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[1],
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[2],
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[3],
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[4],
				(int) ((unsigned char *) req->ifr_hwaddr.sa_data)[5]);

			hardwareAddress = (*env)->NewStringUTF( env, (const char*) formatted );
			free( formatted );
		}
		/* close the socket */
		close(sd);
		/* We are done with the string - let the VM at it . */
		(*env)->ReleaseStringChars(env, name, interfaceName);
		/* free the memory used for the request structure */
		free( req );
	}

	/* free memory used for strings */
	free( interfaceName );

	/* return the hardware address */
	return hardwareAddress;
}


JNIEXPORT
jstring JNICALL 
Java_com_instant802_cldc_linux_NetInterface_nativeGetIpAddress(JNIEnv *env, jobject obj, jbyteArray name, jint len)
{
	int retVal = 1;
	int fd;
	struct ifreq req;
	int i;
	jstring sname;
	
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if(fd<0)
	{
		// fprintf(stderr, "***************** error no socket\r\n");
		retVal = 0;
	}
	else
	{
		memset(&req, 0, sizeof(req));
		for (i = 0; i< len; i++)
		{
			req.ifr_name[i] =  (*env)->GetByteArrayElements(env, name, 0)[i];
		}

		req.ifr_addr.sa_family = AF_INET;
		if(ioctl(fd, SIOCGIFADDR, &req) < 0)
		{
			// fprintf(stderr, "*********************** error no ip address %s\r\n", req.ifr_name);
			retVal = 0;
		}
		close(fd);
	}

	if(retVal)
	{
		sname = (*env)->NewStringUTF( env, (const char*) inet_ntoa(((struct sockaddr_in*)&req.ifr_addr)->sin_addr) );
	} else {
		sname = NULL;
	}
	return sname;
}

/* getNetMask( byte[], len ) */
JNIEXPORT 
jstring JNICALL
Java_com_instant802_cldc_linux_NetInterface_nativeGetNetmask(JNIEnv *env, jobject obj,  jbyteArray address, jint len)
{
	int retVal = 1;
	int fd;
	struct ifreq req;
	int i;
	jstring mask;
	
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if(fd<0)
	{
		//printf("error no socket\r\n");
		retVal = 0;
	}
	else
	{
		memset(&req, 0, sizeof(req));
		for (i = 0; i< len; i++)
		{
			req.ifr_name[i] =  (*env)->GetByteArrayElements(env, address, 0)[i];
		}

		req.ifr_addr.sa_family = AF_INET;
		if(ioctl(fd, SIOCGIFNETMASK, &req) < 0)
		{
			//printf("error %s\r\n", req.ifr_name);
			retVal = 0;
		}
		close(fd);
	}

	if(retVal)
	{
		 mask = (*env)->NewStringUTF( env, (const char*) inet_ntoa(((struct sockaddr_in*)&req.ifr_addr)->sin_addr) );
	} else {
		mask = NULL;
	}
	return mask;
}
