//
// de.gyronet.kangaroo.io.InetImpl.java
// kangaroo
//
// Created by Marc Kleine-Budde on 04.04.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java.net.InetAddress.java
//			      java.net.PlainSocketImpl.java
//			      java.net.SocketInputStream.java
//			      java.net.SocketOutputStream.java
// from the Kaffe Project
//
// see original Copyright below... 

/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package de.gyronet.kangaroo.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.NumberFormatException;
import javax.microedition.io.SocketConnection;

public class InetImpl extends AbstractFileDescriptor {

    protected int addr;
    protected int port;
    protected int localaddr;
    protected int localport;
    protected int timeout;

    private byte[] buf = new byte[1];

    protected InetImpl() {
    }


    protected native void close0() throws IOException;


    protected int read() throws IOException {
	if (readBytes(buf, 0, 1) == 1) {
	    return (buf[0] & 0xFF);
	}
	return (-1);
    }

    protected native int readBytes(byte[] buf, int off, int len) throws IOException;


    protected void write(int b) throws IOException {
	buf[0] = (byte)b;
	writeBytes(buf, 0, 1);
    }

    protected native void writeBytes(byte[] buf, int off, int len) throws IOException;


    protected native int available0() throws IOException;


    protected String getAddress() throws IOException {
	ensureOpen();

	return (ip2Str(addr));
    }
    protected String getLocalAddress() throws IOException {
	ensureOpen();

	return (ip2Str(localaddr));
    }

    protected int getLocalPort() throws IOException {
	ensureOpen();

	return (localport);
    }

    protected int getPort() throws IOException {
	ensureOpen();

	return (port);
    }

    public static String ip2Str(int val) {
	return (((int) ((val >> 24) & 0xff)) + "." +
		((int) ((val >> 16) & 0xff)) + "." +
		((int) ((val >>  8) & 0xff)) + "." +
		((int) ((val      ) & 0xff))
		);
    }

    protected int getSocketOption(byte option) throws IllegalArgumentException, IOException {
	ensureOpen();

	switch (option) {
	case SocketConnection.SNDBUF:
	case SocketConnection.RCVBUF:
	case SocketConnection.LINGER:
	case SocketConnection.DELAY:
	    break;
	default:
		throw new IllegalArgumentException("Unknown socket option");
	}

	return getSocketOption0(option);
    }

    protected void setSocketOption(byte option, int value) throws IllegalArgumentException, IOException {
	ensureOpen();

	switch (option) {
	case SocketConnection.SNDBUF:
	case SocketConnection.RCVBUF:
	case SocketConnection.LINGER:
	    break;
	case SocketConnection.DELAY:
	    if (value < 0) {
		throw new IllegalArgumentException("delay < 0");
	    }
	    break;
	default:
	    throw new IllegalArgumentException("Unknown socket option");
	}

	setSocketOption0(option, value);
    }

    protected native void create0(boolean stream) throws IOException;
    protected native void connect0(int address, int port) throws IOException;

    protected native int getSocketOption0(byte option) throws IOException;
    protected native void setSocketOption0(byte option, int value) throws IOException;

    protected native void acceptAndOpen0(InetImpl con) throws IOException;
    protected native void bind0(int addr, int port) throws IOException;
    protected native void listen0(int backlog) throws IOException;
}
