//
// de.gyronet.kangaroo.io.JarConnection.java
// kangaroo
//
// Created by Marc Kleine-Budde on 03.31.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java/util/zip/ZipFile.java
// from the Kaffe Project
//
// see original Copyright below... 

/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package de.gyronet.kangaroo.io;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.microedition.io.InputConnection;
import kaffe.util.Ptr;
import de.gyronet.kangaroo.io.FileConnection;

/**
 * @version $Id: JarConnection.java,v 1.1.1.1 2003/10/31 00:47:08 carter Exp $
 * @author Marc Kleine-Budde
 *
 */
public class JarConnection implements InputConnection, Protocol {

    private byte[] buf;
    private boolean open = false;

    private Ptr jar;
    private long size;
    private int method;
    private long csize;
    private long offset;

    /*
     * the format of the target string is excepted as:
     *
     * [jarfile][path-separator][resource]
     *
     * e.g. for *nix systems: '/path/to/foo.jar:bar.txt'
     */
    public Connection open(String target, Hashtable params, int mode, boolean timeouts) throws IOException {
        if (mode != Connector.READ) {
	    throw new IOException("illegal mode: " + mode);
	}

	int index = target.indexOf(FileConnection.pathSeparator);
	if (index < 1) {
	    throw new IOException("could not parse scheme: " + target);
	}

	String jarFile = target.substring(0, index);
	String resource = target.substring(index + 1);

	/*
	 * opens the jar-file...returns a pointer to it
	 */
	jar = open0(jarFile);
	
	if (jar == null) {
	    throw new IOException("Jar-file " + jarFile + " not found");
	}

	/*
	 * opens the entry in the jar file, and sets the variables size, method, csize, offset
	 */
	getResource0(jar, resource);

	/*
	 * read data into byte array, make input stream of it and close jar file
	 */
	buf = getJarData0(jar);

	close0(jar);
	jar = null;

	open = true;
	
	return this;
    }



    public void close() throws IOException {
	open = false;
	buf = null;
    }

    public InputStream openInputStream() throws IOException {
	ensureOpen();

        return(new ByteArrayInputStream(buf));
    }

    public DataInputStream openDataInputStream() throws IOException {
        return(new DataInputStream(openInputStream()));
    }

    protected void ensureOpen() throws IOException {
        if (!open) {
            throw new IOException("Connection closed");
        }
    }

    private native Ptr open0(String jarFile) throws IOException;
    private native void getResource0(Ptr jar, String resource) throws IOException;
    private native byte[] getJarData0(Ptr jar);
    private native void close0(Ptr jar) throws IOException;
}
