//
// // de.gyronet.kangaroo.io.URL.java
// kangaroo
//
// Created by Marc Kleine-Budde on 04.22.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from: java.net.InetAddress.java
// from the Kaffe Project
//
// see original Copyright below...

/*
 * Java core library component.
 * 
 * Copyright (c) 1997, 1998 Transvirtual Technologies, Inc. All rights
 * reserved.
 * 
 * See the file "license.terms" for information on usage and redistribution of
 * this file.
 */

package de.gyronet.kangaroo.io;

import java.lang.IllegalArgumentException;
import java.io.IOException;

public class URL
{
	public int addr;
	public int port;
	public String host;

	private URL()
	{
	}

	private URL(int addr, int port)
	{
		this.addr = addr;
		this.port = port;
	}
	
	private URL(int addr, int port, String host) {
		this.addr = addr;
		this.port = port;
		this.host = host;
	}

	public static URL parseURL(String target)
		throws IllegalArgumentException, IOException
	{
		int addr, port;
		String host;

		if (target.charAt(0) != '/' || target.charAt(1) != '/')
		{
			throw new IllegalArgumentException("Protocol must start with \"//\"");
		}

		int index = target.indexOf(':');

		if (index < 0 || index == target.length())
		{
			throw new IllegalArgumentException("no port number");
		}
		else if (index != 2)
		{
			host = target.substring(2, index);
			addr = getByName(host);
		}
		else
		{
			addr = 0;
			host = null;
		}

		try
		{
			port = Integer.parseInt(target.substring(index + 1));
		}
		catch (NumberFormatException _)
		{
			throw new IllegalArgumentException("bad port number");
		}

		if (port < 0 || port > 65535)
		{
			throw new IOException("illegal port number" + port);
		}

		return new URL(addr, port, host);
	}

	public static int getByName(String host) throws IOException
	{
		int ip, index;

		if (!Character.isDigit(host.charAt(0)))
		{
			ip = lookupHostAddr0(host);
		}
		else
		{
			ip = 0;

			while ((index = host.indexOf('.')) > 0)
			{
				String s = host.substring(0, index);
				host = host.substring(index + 1);

				ip <<= 8;

				try
				{
					ip |= Integer.parseInt(s);
				}
				catch (NumberFormatException _)
				{
				}
			}
		}

		return ip;
	}

	private static native int lookupHostAddr0(String host) throws IOException;
}
