/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998, 1999
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.lang;

public final class Character {

    public static final int MIN_RADIX = 2;
    public static final int MAX_RADIX = 36;
    public static final char MIN_VALUE = 0x0000;
    public static final char MAX_VALUE = 0xffff;

    private final char value;

    public static final Class TYPE = Class.getPrimitiveClass("char");

    public Character(char value) {
	this.value = value;
    }

    public char charValue() {
	return (value);
    }

    public static int digit(char ch, int radix) {
        int value = -1;
        if (radix >= Character.MIN_RADIX && radix <= Character.MAX_RADIX) {
            if (isDigit(ch)) 
		value = ch - '0';
            else if (isUpperCase(ch) || isLowerCase(ch))
		value = (ch & 0x1F) + 9;
        }
        return (value < radix) ? value : -1;
    }

    public boolean equals(Object obj) {
	return (obj instanceof Character)
	    && (((Character) obj).value == this.value);
    }

    public int hashCode() {
	return ((int)value);
    }

    public static boolean isDigit(char ch) {
        return ch >= '0' && ch <= '9';
    }

    public static boolean isLowerCase(char ch) {
        return ch >= 'a' && ch <= 'z';
    }

    public static boolean isUpperCase(char ch) {
        return ch >= 'A' && ch <= 'Z';
    }

    public static char toLowerCase(char ch) {
        if (isUpperCase(ch)) 
	    return (char)(ch + ('a' - 'A'));
        else return ch;
    }

    public static boolean isWhitespace(char ch) {
    	return
			ch == ' ' ||
			ch == '\n' ||
			ch == '\t' ||
			ch == '\r';
    }
    
    public String toString() {
	return (String.valueOf(value));
    }

    public static char toUpperCase(char ch) {
        if (isLowerCase(ch)) return (char)(ch - ('a' - 'A'));
        else return ch;
    }
}

