/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.lang;

final public class Integer {

    private int value;
    final public static int MIN_VALUE = 0x80000000;
    final public static int MAX_VALUE = 0x7fffffff;
    final public static Class TYPE = Class.getPrimitiveClass("int");

    final static char[] digits = {
        '0' , '1' , '2' , '3' , '4' , '5' ,
        '6' , '7' , '8' , '9' , 'a' , 'b' ,
        'c' , 'd' , 'e' , 'f' , 'g' , 'h' ,
        'i' , 'j' , 'k' , 'l' , 'm' , 'n' ,
        'o' , 'p' , 'q' , 'r' , 's' , 't' ,
        'u' , 'v' , 'w' , 'x' , 'y' , 'z'
    };

    public Integer(int value) {
	this.value = value;
    }	

    public byte byteValue() {
        return (byte)value;
    }

    public boolean equals(Object obj) {
	return (obj instanceof Integer) &&
	    (((Integer)obj).value == this.value);
    }

    public int hashCode() {
	return value;
    }

    public int intValue() {
	return value;
    }

    public long longValue() {
	return (long)value;
    }

    public static int parseInt(String s) throws NumberFormatException {
	return (parseInt(s, 10));
    }

    public static int parseInt(String s, int radix) throws NumberFormatException
    {
	int result = 0;
	int max = s.length();
	int pos, sign;

	if (s == null || s.length() <= 0) {
	    throw new NumberFormatException(s);
	}

	/* Check for negativity */
	if (s.charAt(0) == '-') {
	    sign = -1;
	    pos = 1;
	} else {
	    sign = 1;
	    pos = 0;
	}

	for (; pos < max; pos++) {
	    int digit = Character.digit(s.charAt(pos), radix);
	    if (digit == -1) {
		throw new NumberFormatException(s);
	    }
	    int nresult = (result * radix) + (sign * digit);
	    if (((sign == 1) && (result > nresult)) ||
		((sign == -1) && (result < nresult)) )
		{
		    throw new NumberFormatException(s);
		}
	    result = nresult;
	}
	return (result);
    }

    public short shortValue() {
        return (short)value;
    }

    public static String toBinaryString(int i) {
	return toUnsignedString(i, 1);
    }

    public static String toHexString(int i) {
	return toUnsignedString(i, 4);
    }

    public static String toOctalString(int i) {
	return toUnsignedString(i, 3);
    }

    public String toString() {
	return toString(this.value);
    }

    public static String toString(int i) {
	return toString(i, 10);
    }

    public static String toString(int i, int radix) {
        if (radix < Character.MIN_RADIX || radix > Character.MAX_RADIX)
	    radix = 10;

        char buf[] = new char[33];
        boolean negative = (i < 0);
        int charPos = 32;

        if (!negative) i = -i;

        while (i <= -radix) {
            buf[charPos--] = digits[-(i % radix)];
            i = i / radix;
        }
        buf[charPos] = digits[-i];

        if (negative) buf[--charPos] = '-';

        return new String(buf, charPos, (33 - charPos));
    }

    private static String toUnsignedString(int i, int shift) {
        char[] buf = new char[32];
        int charPos = 32;
        int radix = 1 << shift;
        int mask = radix - 1;
        do {
            buf[--charPos] = digits[i & mask];
            i >>>= shift;
        } while (i != 0);

        return new String(buf, charPos, (32 - charPos));
    }

    public static Integer valueOf(String s) throws NumberFormatException
    {
	return valueOf(s, 10);
    }

    public static Integer valueOf(String s, int radix) throws NumberFormatException
    {
	return new Integer(parseInt(s, radix));
    }
}