/* Header for error handling for i802 ui related functions. */

#ifndef _i802_err_h_
#define _i802_err_h_ 1

#include "debug.h"
#include <errno.h> 

#define i802_SUCCESS 0

// use this to raise an error and pass it up to your caller.  parameters are as for printf
#define i802_RAISEF( format, ... ) \
  return i802_err_raise( __PRETTY_FUNCTION__,__FILE__,__LINE__,format, __VA_ARGS__ )

// use this to re-raise an error and pass it up to your caller.  parameters are as for printf
#define i802_RERAISEF( error, format, ... ) \
  return i802_err_raise( error->function,error->file,error->lineno,format, __VA_ARGS__ )

#define i802_RAISE( message ) \
  return i802_err_raise( __PRETTY_FUNCTION__,__FILE__,__LINE__, "%s", message )

// use this to pass an error generated by a called function up to your caller.
#define i802_PASS( e ) \
 { \
   i802_err *__errptr = (i802_err *) (e); \
  if( __errptr ) { return __errptr; }\
 }

#define i802_SYS( e ) \
  if( (e) == -1 ) { \
      return i802_err_raise( __PRETTY_FUNCTION__,__FILE__,__LINE__, " system errno=%d", errno );\
  }     


#define i802_LOG( e ) \
  i802_err_print( e ); \
  
typedef struct _i802_err{
  char* message;
  char* file;
  char* function;
  int lineno;
} i802_err;

i802_err* i802_err_raise( char* function, char* file, int lineno, char* format, ... );

int i802_err_free( i802_err* error );

char* i802_err_asstring( i802_err* error );

int i802_print_error( i802_err* error );

#endif

