
#ifndef __LCL_COUNTERS_H___
#define __LCL_COUNTERS_H____

#include<stdarg.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "xmlutils.h"
#include "counters.h"

/* columns present in counters tables */
#define COUNTERS_TRANS_FRAG_COUNT               "transmitted-fragment-count"
#define COUNTERS_MULTI_TRANS_FRAME_COUNT        "multicast-transmitted-frame-count"
#define COUNTERS_FAILED_COUNT                   "failed-count"
#define COUNTERS_RETRY_COUNT                    "retry-count"
#define COUNTERS_MULTIPLE_RETRY_COUNT           "multiple-retry-count"
#define COUNTERS_FRAME_DUPLICATE_COUNT          "frame-duplicate-count"
#define COUNTERS_RTS_SUCCESS_COUNT              "rts-success-count"
#define COUNTERS_RTS_FAILURE_COUNT              "rts-failure-count"
#define COUNTERS_ACK_FAILURE_COUNT              "ack-failure-count"
#define COUNTERS_RECV_FRAGMENT_COUNT            "received-fragment-count"
#define COUNTERS_MULTI_RECV_FRAME_COUNT         "multicast-received-frame-count"
#define COUNTERS_FCS_ERROR_COUNT                "fcs-error-count"
#define COUNTERS_TRANS_FRAME_COUNT              "transmitted-frame-count"
#define COUNTERS_WEP_UNDECRYPT_COUNT            "wep-undecryptable-count"

struct counters_data_info{
	char *tx_frag_cnt;
	int tx_frag_cnt_len;
	char *multi_tx_frame_cnt;
	int multi_tx_frame_cnt_len;
	char *failed_cnt;
	int failed_cnt_len;
	char *retry_cnt;
	int retry_cnt_len;
	char *multi_retry_cnt;
	int multi_retry_cnt_len;
	char *frame_dup_cnt;
	int frame_dup_cnt_len;
	char *rts_succ_cnt;
	int rts_succ_cnt_len;
	char *rts_fail_cnt;
	int rts_fail_cnt_len;
	char *ack_fail_cnt;
	int ack_fail_cnt_len;
	char *rx_frag_cnt;
	int rx_frag_cnt_len;
	char *multi_rx_frame_cnt;
	int multi_rx_frame_cnt_len;
	char *fcs_error_cnt;
	int fcs_error_cnt_len;
	char *tx_frame_cnt;
	int tx_frame_cnt_len;
	char *wep_undec_cnt;
	int wep_undec_cnt_len;
};

static int CheckAndRetrieveTransmitFragCountFromXml(xmlDocPtr);
static int CheckAndRetrieveMultiTransmitFrameCountFromXml(xmlDocPtr);
static int CheckAndRetrieveFailedCountFromXml(xmlDocPtr);
static int CheckAndRetrieveRetryCountFromXml(xmlDocPtr);
static int CheckAndRetrieveMultipleRetryCountFromXml(xmlDocPtr);
static int CheckAndRetrieveFrameDupCountFromXml(xmlDocPtr);
static int CheckAndRetrieveRTSSuccessCountFromXml(xmlDocPtr);
static int CheckAndRetrieveRTSFailureCountFromXml(xmlDocPtr);
static int CheckAndRetrieveACKFailureCountFromXml(xmlDocPtr);
static int CheckAndRetrieveRecvFragCountFromXml(xmlDocPtr);
static int CheckAndRetrieveMultiRecvFrameCountFromXml(xmlDocPtr);
static int CheckAndRetrieveFCSErrorCountFromXml(xmlDocPtr);
static int CheckAndRetrieveTransFrameCountFromXml(xmlDocPtr);
static int CheckAndRetrieveWEPUndecryptCountFromXml(xmlDocPtr);

static void lcl_trans_frag_xml(xmlDocPtr, xmlNodePtr);
static void lcl_multi_trans_frame_xml(xmlDocPtr, xmlNodePtr);
static void lcl_failed_xml(xmlDocPtr, xmlNodePtr);
static void lcl_retry_xml(xmlDocPtr, xmlNodePtr);
static void lcl_multiple_retry_xml(xmlDocPtr, xmlNodePtr);
static void lcl_frame_dup_xml(xmlDocPtr, xmlNodePtr);
static void lcl_rts_success_xml(xmlDocPtr, xmlNodePtr);
static void lcl_rts_failure_xml(xmlDocPtr, xmlNodePtr);
static void lcl_ack_failure_xml(xmlDocPtr, xmlNodePtr);
static void lcl_recv_frag_xml(xmlDocPtr, xmlNodePtr);
static void lcl_multi_recv_frame_xml(xmlDocPtr, xmlNodePtr);
static void lcl_fcs_error_xml(xmlDocPtr, xmlNodePtr);
static void lcl_trans_frame_xml(xmlDocPtr, xmlNodePtr);
static void lcl_wep_undecrypt_xml(xmlDocPtr, xmlNodePtr);

static void GetTransmittedFragmentCount(xmlDocPtr, int);
static void GetMultiTransmittedFrameCount(xmlDocPtr, int);
static void GetFailedCount(xmlDocPtr, int);
static void GetRetryCount(xmlDocPtr, int);
static void GetMultipleRetryCount(xmlDocPtr, int);	
static void GetFrameDuplicateCount(xmlDocPtr, int);	
static void GetRTSSuccessCount(xmlDocPtr, int);	
static void GetRTSFailureCount(xmlDocPtr, int);		
static void GetACKFailureCount(xmlDocPtr, int);	
static void GetReceivedFragmentCount(xmlDocPtr, int);	
static void GetMultiReceivedFrameCount(xmlDocPtr, int);
static void GetFCSErrorCount(xmlDocPtr, int);
static void GetTransmittedFrameCount(xmlDocPtr, int);
static void GetWEPUndecryptCount(xmlDocPtr, int);
#endif  /* __LCL_COUNTERS_H__*/
