/* Copyright 2004 Instant802 Networks, Inc. All Rights Reserved. */

var UPPERCASE_LETTER_SET = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
var LOWERCASE_LETTER_SET = "abcdefghijklmnopqrstuvwxyz";
var LETTER_SET = UPPERCASE_LETTER_SET + LOWERCASE_LETTER_SET;
var DIGIT_SET = "0123456789";
var LETTER_DIGIT_SET = LETTER_SET + DIGIT_SET;
var HEX_DIGIT_SET = "0123456789abcedfABCDEF";
var LETTER_DIGIT_HYPHEN_SET = LETTER_DIGIT_SET + "-";

/**
 * Sets all options of a selectBox to true.
 * @param the selectBox to whose options should be set.
 * @return true.
 */
function selectAllItems(selectBox) {
    for (var i = selectBox.options.length-1; i > -1; i--) {
	selectBox.options[i].selected = true;
    }
    return true;
}

/**
 * Check to see if a value appears in the list of options for
 * a select box.
 * @selectBox the box to check
 * @value the value to look for
 * @return true if value is found, false otherwise.
 */
function selectValueExists(selectBox, value) {
    for (var i = selectBox.options.length-1; i > -1; i--) {
        if (selectBox.options[i].value == value) {
            return true;
        }
    }
    return false;
}

function selectItemExists(selectBox, item) {
    return selectValueExists(selectBox, item.value);
}

function addSelectItem(selectBox, item) {
    if (!selectItemExists(selectBox, item)) {
        selectBox.options[selectBox.options.length] =
            new Option(item.value, item.value);
    }
    item.value = "";
}

function insertSelectOption(selectBox, insertIndex, option) {
    for (var i = selectBox.options.length-1; i > -1; i--) {
        var curOption = selectBox.options[i];
        selectBox.options[i+1] = new Option(curOption.text, curOption.value);
        if (i == insertIndex) {
            selectBox.options[i] = option;
            break;
        }
    }
}

function removeSelectItem(selectBox, value) {
    for (var i = selectBox.options.length-1; i > -1; i--) {
        if (selectBox.options[i].value == value) {
            selectBox.options[i] = null;
        }
    }
}

/**
 * Removes the selected items from a selectBox, and toggles its removeButton.
 * @param selectBox the selectBox to remove the selected items of.
 * @param removeButton the button that removes selected items from selectBox.
 */
function removeSelectItems(selectBox, removeButton) {
    if (selectBox.selectedIndex > -1) {
        for (var i = selectBox.options.length-1; i > -1; i--) {
            if (selectBox.options[i].selected) {
                selectBox.options[i] = null;
            }
        }
    }
    toggleSelectRemoveButton(selectBox, removeButton);
}

function toggleSelectRemoveButton(selectBox, removeButton) {
    if (selectBox.selectedIndex > -1) {
        removeButton.disabled = false;
    } else {
        removeButton.disabled = true;
    }
}

/**
 * Determines if we should move to the next box.
 * @param box    the box we're currently on.
 * @param letter the letter that indicates this box is complete.
 * If the last character in box is letter, delete letter, and return true;
 * Otherwise, return false.
 */
function shouldMovetoNext(box, letter) {
    var res = false;
    var lastIndex = box.value.length - 1;
    if (box.value.charAt(lastIndex) == letter) {
        box.value = box.value.substring(0, lastIndex);
        res = true;
    }
    return res;
}

/**
 * Determines if we should move to the next IP box.
 * @param box the box we're currently on.
 * If the last character in box is '.', delete it, and return true;
 * Otherwise, return false.
 */
function shouldIpMoveToNext(box) {
    return false;
    /*return shouldMovetoNext(box, '.');*/
}

/**
 * Determines if we should move to the next MAC box.
 * @param box the box we're currently on.
 * If the last character in box is ':', delete it, and return true;
 * Otherwise, return false.
 */
function shouldMacMoveToNext(box) {
    return false;
    /*return shouldMovetoNext(box, ':');*/
}

/**
 * A function that can be used as an onKeyPress handler to permit only
 * hex characters (and control characters).
 */
function keyPressHexOnly(evt) {
    var e;
    evt = (evt) ? evt : event;
    var charCode = (evt.charCode) ? evt.charCode : ((evt.keyCode) ? evt.keyCode : ((evt.which) ? evt.which : 0));
    if (charCode <= 31) {
        return true;
    }
    var key = String.fromCharCode(charCode);
    if (HEX_DIGIT_SET.indexOf(key) == -1) {
        return false;
    } else {
        return true;
    }
}

/**
 * Checks if a string is a valid hex octet.
 * @param hex the potential hex octet.
 * @return true, if hex is composed of exactly 2 hex digits (case-insensitive);
 *         false, otherwise.
 */
function hexIsValid(hex) {
    var number;
    number = parseInt(hex, 16);

    if (number < 0 || number > 255) {
        return false;
    }
    
    var convertedBack;
    if (number < 16) {
        convertedBack = "0" + number.toString(16);
    } else {
        convertedBack = number.toString(16);
    }

    if (hex.toLowerCase() != convertedBack.toLowerCase()) {
        return false;
    }

    return true;
}

function octetIsValid(octet) {
    if (octet.length > 3) {
        octet = octet.substring(0, 2);
    }

    return isIntInRange(octet, 0, 255);
}

function getInvalidOctetReason(octet) {
    if (octet.length > 3) {
        octet = octet.substring(0, 2);
    }

    return getIntNotInRangeReason(octet, 0, 255);
}

/**
 * Checks if a dotted decimal is valid.
 * @param macro elements macro[0-3] should make up the dotted decimal.
 * @return true if each octet is between 0 and 255.
 */
function dottedDecimalIsValid(macro) {
    for (var i = 0; i < 4; i++) {
        var octet = document.forms[0].elements[macro + i].value;
        if (! octetIsValid(octet)) {
            return false;
        }
    }
    return true;
}

/**
 * Checks if a dotted decimal is blank.
 * @param macro elements macro[0-3] should make up the dotted decimal.
 * @return true if each octet is blank.
 */
function dottedDecimalIsBlank(macro) {
    for (var i = 0; i < 4; i++) {
        var octet = document.forms[0].elements[macro + i].value;
        if (octet.length != 0) {
            return false;
        }
    }
    return true;
}

/**
 * Checks if a dotted decimal is valid or blank
 * @param macro elements macro[0-3] should make up the dotted decimal.
 * @return true if all octets are between 0 and 255 or all octets are blank.
 */
function dottedDecimalIsValidOrBlank(macro) {
    if (dottedDecimalIsBlank(macro) || dottedDecimalIsValid(macro)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Gets a 32-bit integer from a dotted decimal macro.
 * @param macro elements macro[0-3] should make up the dotted decimal.
 * @return a 32-bit integer with macro[0-3] in each octet.
 */
function dottedDecimalToInt(macro) {
    var n = 0;
    for (var i = 0; i < 4; i++) {
        var octet = parseInt(document.forms[0].elements[macro + i].value);

        n = n * 256;
        n = n + octet;
    }
    return n;
}

/**
 * Gets a textual description of why a dotted decimal is invalid.
 * @param macro macro that returned false for dottedDecimalIsValid()
 * @return a textual description of why a dotted decimal is invalid.
 */
function getInvalidDottedDecimalReason(macro) {
    var msg = "";

    if (dottedDecimalIsBlank(macro)) {
        return "Address is blank.";
    }

    for (var i = 0; i < 4; i++) {
        var octet = document.forms[0].elements[macro + i].value;
        if (! octetIsValid(octet)) {
            if (msg.length > 0) {
                msg = msg + " ";
            }

            msg = msg 
                + "Octet " + (i+1) + " is invalid: "
                + getInvalidOctetReason(octet);
        }
    }

    return msg;
}

/**
 * Checks if an IP address is valid.
 * @param macro elements macro[0-3] should make up the IP address.
 * @return true if the IP address is not on local network, a broadcast,
 *              multicast, reserved, or an invalid dotted decimal.
 */
function ipIsValid(macro) {
    if (! dottedDecimalIsValid(macro)) {
        return false;
    }

    var octet0 = parseInt(document.forms[0].elements[macro + 0].value);
    var octet1 = parseInt(document.forms[0].elements[macro + 1].value);
    var octet2 = parseInt(document.forms[0].elements[macro + 2].value);
    var octet3 = parseInt(document.forms[0].elements[macro + 3].value);

    if (octet0 == 0) { return false; } // Local network
    if (octet0 == 255 && octet1 == 255 && octet2 == 255 && octet3 == 255) {
        return false;
    } // Broadcast
    if (octet0 >= 224 && octet0 <= 239) { return false; } // Multicast
    if (octet0 >= 240 && octet0 <= 255) { return false; } // Reserved/ifconfig

    return true;
}

/**
 * Gets a textual description of why an IP address is invalid.
 * @param macro macro that returned false for ipIsValid()
 * @return a textual description of why macro is invalid.
 */
function getInvalidIpReason(macro) {
    var msg = "";

    if (! dottedDecimalIsValid(macro)) {
        return getInvalidDottedDecimalReason(macro);
    }

    var octet0 = parseInt(document.forms[0].elements[macro + 0].value);
    var octet1 = parseInt(document.forms[0].elements[macro + 1].value);
    var octet2 = parseInt(document.forms[0].elements[macro + 2].value);
    var octet3 = parseInt(document.forms[0].elements[macro + 3].value);

    if (octet0 == 0) { 
        return "Address is on local network (0.x.x.x).";
    }

    if (octet0 == 255 && octet1 == 255 && octet2 == 255 && octet3 == 255) {
        return "Address is limited broadcast (255.255.255.255).";
    }

    if (octet0 >= 224 && octet0 <= 239) {
        return "Address is multicast (224.x.x.x - 239.x.x.x).";
    }

    if (octet0 >= 240 && octet0 <= 255) {
        return "Address is reserved for future use (240.x.x.x - 255.x.x.x).";
    }

    return "";
}

/**
 * Checks if an IP address is valid and not a loopback.
 * @param macro elements macro[0-3] should make up the IP address.
 * @return true if the IP address is not on local network, a broadcast,
 *              multicast, reserved, a loopback, or an invalid dotted decimal.
 */
function ipIsValidNotLoopback(macro) {
    if (! ipIsValid(macro)) {
        return false;
    }

    var octet0 = parseInt(document.forms[0].elements[macro + 0].value);

    if (octet0 == 127) { return false; } // Loopback

    return true;
}

/**
 * Gets a textual description of why an IP address is invalid or a loopback.
 * @param macro macro that returned false for ipIsValidNotLoopback()
 * @return a textual description of why macro is invalid.
 */
function getInvalidIpOrLoopbackReason(macro) {
    if (! ipIsValid(macro)) {
        return getInvalidIpReason(macro);
    }

    var octet0 = parseInt(document.forms[0].elements[macro + 0].value);

    if (octet0 == 127) {
        return "Address is loopback (127.x.x.x).";
    }

    return "";
}

/**
 * Checks if an IP address is valid and not a loopback, or blank.
 * @param macro elements macro[0-3] should make up the IP address.
 * @return true if the IP address is not on local network, a broadcast,
 *              multicast, reserved, a loopback, or an invalid dotted decimal,
 *              or all the octets are blank.
 */
function ipIsValidOrBlankNotLoopback(macro) {
    if (dottedDecimalIsBlank(macro) || ipIsValidNotLoopback(macro)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Gets a textual description of why an IP address is invalid or a loopback.
 * @param macro macro that returned false for ipIsValidOrBlankNotLoopback()
 * @return a textual description of why macro is invalid.
 */
function getInvalidIpOrBlankNotLoopbackReason(macro) {
    return getInvalidIpOrLoopbackReason(macro);
}

/**
 * Checks if an network mask is valid.
 * @param macro elements macro[0-3] should make up the network mask.
 * @return true if the network mask is contiguous and a valid dotted decimal.
 */
function maskIsValid(macro) {
    var ip = dottedDecimalToInt(macro);

    if (ip == 0x00000000 || ip == 0x80000000 || ip == 0xC0000000 || ip == 0xE0000000 ||
        ip == 0xF0000000 || ip == 0xF8000000 || ip == 0xFC000000 || ip == 0xFE000000 ||
        ip == 0xFF000000 || ip == 0xFF800000 || ip == 0xFFC00000 || ip == 0xFFE00000 ||
        ip == 0xFFF00000 || ip == 0xFFF80000 || ip == 0xFFFC0000 || ip == 0xFFFE0000 ||
        ip == 0xFFFF0000 || ip == 0xFFFF8000 || ip == 0xFFFFC000 || ip == 0xFFFFE000 ||
        ip == 0xFFFFF000 || ip == 0xFFFFF800 || ip == 0xFFFFFC00 || ip == 0xFFFFFE00 ||
        ip == 0xFFFFFF00 || ip == 0xFFFFFF80 || ip == 0xFFFFFFC0 || ip == 0xFFFFFFE0 ||
        ip == 0xFFFFFFF0 || ip == 0xFFFFFFF8 || ip == 0xFFFFFFFC || ip == 0xFFFFFFFE ||
        ip == 0xFFFFFFFF) {
        return true;
    } else {
        return false;
    }
}

/**
 * Gets a textual description of why a mask is invalid.
 * @param macro macro that returned false for maskIsValid()
 * @return a textual description of why macro is invalid.
 */
function getInvalidMaskReason(macro) {
    if (! maskIsValid(macro)) {
        return "Mask is non-contiguous.";
    } else {
        return "";
    }
}

/**
 * Checks if an IP address and network mask are valid together.
 * They should each be validated separately first.
 * @param ipMacro elements macro[0-3] should make up the IP address.
 * @param maskMacro elements macro[0-3] should make up the network mask.
 * @return true if the IP address is not on the local network, a broadcast,
 *              or a network number.
 */
function ipAndMaskAreValidTogether(ipMacro, maskMacro) {
    var ip = dottedDecimalToInt(ipMacro);
    var mask = dottedDecimalToInt(maskMacro);

    if (mask != 0 && (ip & mask) == 0) { return false; } // On local network (RFC1700)
    if ((ip & ~mask) == ~mask) { return false; } // Broadcast
    if (~mask != 0 && (ip & ~mask) == 0) { return false; } // Network number

    return true;
}

/**
 * Gets a textual description of why an IP address and network mask are
 * invalid together.
 * @param ipMacro macro that returned false for ipAndMaskAreValidTogether().
 * @param maskMacro macro that returned false for ipAndMaskAreValidTogether().
 * @return a textual description of why macros are invalid together.
 */
function getInvalidIpAndMaskTogetherReason(ipMacro, maskMacro) {
    var ip = dottedDecimalToInt(ipMacro);
    var mask = dottedDecimalToInt(maskMacro);

    if (mask != 0 && (ip & mask) == 0) {
        return "Address is on local network (network part is all zeros).";
    }

    if ((ip & ~mask) == ~mask) {
        return "Address is directed broadcast (host part is all ones).";
    }

    if (~mask != 0 && (ip & ~mask) == 0) {
        return "Address is network number (host part is all zeros).";
    }

    return "";
}

/**
 * Checks if an IP address and gateway are valid together.
 * They should each be validated separately first.
 * @param ipMacro elements macro[0-3] should make up the IP address.
 * @param maskMacro elements macro[0-3] should make up the network mask.
 * @param gatewayMacro elements macro[0-3] should make up the gateway.
 * @return true if the IP address and gateway are on the same subnet and
 *              not the same address.
 */
function ipAndGatewayAreValidTogether(ipMacro, maskMacro, gatewayMacro) {
    var ip = dottedDecimalToInt(ipMacro);
    var mask = dottedDecimalToInt(maskMacro);
    var gateway = dottedDecimalToInt(gatewayMacro);

    if ((ip & mask) != (gateway & mask)) { return false; } // Not on same subnet
    if (ip == gateway) { return false; } // Same address

    return true;
}

/**
 * Gets a textual description of why an IP address and gateway are
 * invalid together.
 * @param ipMacro macro that returned false for ipAndGatewayAreValidTogether().
 * @param maskMacro macro that returned false for ipAndGatewayAreValidTogether().
 * @param gatewayMacro macro that returned false for ipAndGatewayAreValidTogether().
 * @return a textual description of why macros are invalid together.
 */
function getInvalidIpAndGatewayTogetherReason(ipMacro, maskMacro, gatewayMacro) {
    var ip = dottedDecimalToInt(ipMacro);
    var mask = dottedDecimalToInt(maskMacro);
    var gateway = dottedDecimalToInt(gatewayMacro);

    if ((ip & mask) != (gateway & mask)) {
        return "IP Address and Gateway are not on the same subnet.";
    }
    if (ip == gateway) {
        return "IP Address and Gateway are the same.";
    }

    return "";
}

/**
 * Checks if a server address is valid, given a host ip and mask.
 * They should each be validated separately first.
 * @param serverMacro elements macro[0-3] should make up the server address.
 * @param ipMacro elements macro[0-3] should make up the host IP address.
 * @param maskMacro elements macro[0-3] should make up the network mask.
 * @return true if the server address is on the same subnet, and not
 *              a broadcast/network number, or if the server address is on
 *              a different subnet.
 */
function serverIsValid(serverMacro, ipMacro, maskMacro) {
    var server = dottedDecimalToInt(serverMacro);
    var ip = dottedDecimalToInt(ipMacro);
    var mask = dottedDecimalToInt(maskMacro);

    if ((ip & mask) != (server & mask)) { return true; } // Not on same subnet

    return ipAndMaskAreValidTogether(serverMacro, maskMacro);
}

/**
 * Gets a textual description of why a server address is invalid.
 * @param serverMacro macro that returned false for serverIsValid().
 * @param ipMacro macro that returned false for serverIsValid().
 * @param maskMacro macro that returned false for serverIsValid().
 * @return a textual description of why a server address is invalid.
 */
function getInvalidServerReason(serverMacro, ipMacro, maskMacro) {
    return getInvalidIpAndMaskTogetherReason(serverMacro, maskMacro);
}

function portIsValid(port) {
    return isIntInRange(port, 0, 65535);
}

/**
 * Tabs to an element.
 * @elementName the name of an element to tab to or after.
 * @after       if true, tables the element after elementName;
 *              otherwise, tables to elementName.
 */
function tabTo(elementName, after) {
    for (var i = 0; i < document.forms[0].elements.length; i++) {
        if (document.forms[0].elements[i].name == elementName) {
            if (after) {
                i++;
            }
            document.forms[0].elements[i].focus();
            document.forms[0].elements[i].select();
            break;
        }
    }
}

function validateIpOnChange(box, lastOctet) {
    var shouldTab = false;

    if (!lastOctet) {
        shouldTab = shouldIpMoveToNext(box);
    }
    if (box.value.length != 0 &&
        octetIsValid(box.value) &&
        shouldTab) {
            tabTo(box.name, true);
    }
}

function validateIpOnLeave(box) {
/*
    if (!octetIsValid(box.value)) {
        tabTo(box.name, false);
    }
*/
}

function onMacKeyUp(box, evt) {
    evt = (evt) ? evt : event;
    var charCode = (evt.charCode) ? evt.charCode : ((evt.keyCode) ? evt.keyCode : ((evt.which) ? evt.which : 0));
    if (charCode > 31 && box.value.length == 2) {
        tabTo(box.name, true);
    }
}

function toggleElements(elements, disable) {
    for (var i = 0; i < elements.length; i++) {
        elements[i].disabled = disable;
    }
}

function enableElements(elements) {
    toggleElements(elements, false);
}

function disableElements(elements) {
    toggleElements(elements, true);
}


function getStyleByName(name) {
    return getStyleByLayer(getLayer(name));
}

function getStyleByLayer(element) {
    return element.style;
}

function getLayer(name) {
    if (document.getElementById) {
        return document.getElementById(name);
    } else if (document.all) {
        return document.all[name];
    } else if (document.layers) {
        return document.layers[name];
    }
}


function toggleLayerVisibility(layerID){
    var style = getStyleByName(layerID);

    //if(style.display == "none"){
    if(style.visibility == "hidden"){
        //style.display="";
        style.visibility="visible";
    }else{
        //style.display="none";
        style.visibility="hidden";
    }
    // Mozilla fails to validate the layout after elements are visually removed, thus hack...
	//layoutContent();
}

function updateLayerVisibility(layerID, visible){
    var style = getStyleByName(layerID);

    if (visible) {
        style.display="";
        style.visibility="visible";
    } else {
        style.display="none";
        style.visibility="hidden";
    }
    // Mozilla fails to validate the layout after elements are visually removed, thus hack...
	//layoutContent();
}


function addArbitraryChannels(channelField, channelArray) {
    for (var i = 0; i < channelArray.length; i++) {
        channelField.options[channelField.options.length] = new Option(channelArray[i], channelArray[i]);
    }
}

function resetChannels(channelField) {
    for (var i = channelField.options.length-1; i >= 0; i--) {
        channelField.options[i] = null;
    }
}

/**
 * Updates a channel field.
 * @param modeField        The mode (A/B/G/Turbo) field name.
 * @param channelField     The channel field.
 * @param selectedValue    The currently selected channel.
 * Sets channelField to contain the channels associated with modeField.
 * Selects an appropriate value for channelField, trying selectedValue
 * first.
 */
function updateChannelField(modeFieldName, channelField, selectedValue) {
    resetChannels(channelField);
    var mode = document.forms[0].elements[modeFieldName].value;
    if (mode == 'IEEE 802.11a') {
        addArbitraryChannels(channelField, ieee80211aChannels);
    } else if (mode == 'IEEE 802.11g') {
        addArbitraryChannels(channelField, ieee80211gChannels);
    } else if (mode == 'Atheros Turbo 5 GHz') {
        addArbitraryChannels(channelField, AtherosTurboChannels);
    } else if (mode == 'Atheros Dynamic Turbo 5 GHz') {
        addArbitraryChannels(channelField, AtherosDynTurboChannels);
    } else if (mode == 'Atheros Turbo 2.4 GHz') {
        addArbitraryChannels(channelField, AtherosTurboGChannels);
    } else if (mode == 'Atheros Dynamic Turbo 2.4 GHz') {
        addArbitraryChannels(channelField, AtherosDynTurboGChannels);
    } else if (mode == 'IEEE 802.11b') {
        addArbitraryChannels(channelField, ieee80211bChannels);
    }

    if (selectValueExists(channelField, selectedValue)) {
        selectItem(channelField, selectedValue);
    } else if (selectValueExists(channelField, 'Auto')) {
        selectItem(channelField, 'Auto');
    } else if (channelField.options.length > 0) {
        channelField.options[0].selected = true;
    }
}


/**
 * Updates the list of available hardware modes.

 * @param modeField        The hardware mode (A/B/G/Turbo) field.
 *
 * Removes modes from the list of available modes if there are no
 * channels available for that hardware mode.
 */
function updateModeField(modeField) {
    
    if (ieee80211aChannels.length < 1) {
	removeSelectItem(modeField, 'IEEE 802.11a');
    }
    if (ieee80211bChannels.length < 1) {
	removeSelectItem(modeField, 'IEEE 802.11b');
    }
    if (ieee80211gChannels.length < 1) {
	removeSelectItem(modeField, 'IEEE 802.11g');
    }
    if (AtherosTurboChannels.length < 1) {
	removeSelectItem(modeField, 'Atheros Turbo 5 GHz');
    }
    if (AtherosTurboGChannels.length < 1) {
	removeSelectItem(modeField, 'Atheros Turbo 2.4 GHz');
    }
    if (AtherosDynTurboChannels.length < 1) {
	removeSelectItem(modeField, 'Atheros Dynamic Turbo 5 GHz');
    }
    if (AtherosDynTurboGChannels.length < 1) {
	removeSelectItem(modeField, 'Atheros Dynamic Turbo 2.4 GHz');
    }
}

function selectItem(selectBox, valueToSelect) {
    for (var i = selectBox.options.length-1; i > -1; i--) {
        if (selectBox.options[i].value == valueToSelect) {
            selectBox.options[i].selected = true;
            break;
        }
    }
}

function getSelectedRadioItem(radio) {
    for (var i = 0; i < radio.length; i++) {
        if (radio[i].checked) {
            return radio[i];
        }
    }
}

function addQuadElements(elements, start, prefix) {
    for (var i = 0; i < 4; i++) {
        var elementName =  prefix + i;
        elements[start+i] = document.forms[0].elements[elementName];
    }
}

function addMacElements(elements, start, prefix) {
    for (var i = 0; i < 6; i++) {
        var elementName =  prefix + i;
        elements[start+i] = document.forms[0].elements[elementName];
    }
}

/**
 * Makes the selected layer visible, and unselected layers invisible.
 * @param selectBox    the select box to base the visibility on.
 * @param selectValues An array of values in selectBox.
 * @param layers       An array of layer names corresponding to selectValues.
 */
function resetLayersBySelect(selectBox, selectValues, layers) {
    for (var i = 0; i < layers.length; i++) {
        if (selectBox.value == selectValues[i]) {
            updateLayerVisibility(layers[i], true);
        } else {
            updateLayerVisibility(layers[i], false);
        }
    }
}

function hideLayers(layers) {
    for (var i = 0; i < layers.length; i++) {
        updateLayerVisibility(layers[i], false);
    }
}

function findElementByNameAndValue(name, value) {
    for (var i = 0; i < document.forms[0].elements.length; i++) {
        if (document.forms[0].elements[i].name == name &&
            document.forms[0].elements[i].value == value) {
            return document.forms[0].elements[i];
        }
    }
}

/* solve lame IE problem where you can't access the element directly by
 * name if the name was changed programmatically
 */
function getElementByName(name) {
    for (var i = 0; i < document.forms[0].elements.length; i++) {
        if (document.forms[0].elements[i].name == name) {
            return document.forms[0].elements[i];
        }
    }
}

function addIpElements(elements, start, prefix) {
    addQuadElements(elements, start, prefix + 'Ip');
    addQuadElements(elements, start+4, prefix + 'Netmask');
}

/**
 * Returns a value representing an element's state.
 * @param the element.
 * @return a value representing an element's state.
 */
function extractElementState(element) {
    var state;

    state = element.checked + element.value;

    if (element.options) {
        for (var j = 0; j < element.options.length; j++) {
            state = state + j + ':' + element.options[j];
        }
    }

    return state;
}

/**
 * Returns an Array that contains the form elements state.
 * @param indices an Array with the indices of the form elements.
 * @return an Array that contains the form elements state.
 */
function saveFormElements(indices) {
    var oldElements = new Array(indices.length);
    for (var j = 0; j < indices.length; j++) {
        oldElements[j] = extractElementState(document.forms[0].elements[indices[j]]);
    }
    return oldElements;
}

/**
 * Checks if the elements of a form have changed.
 * @param oldElements the old form.
 * @param indices an Array with the indices of the form elements.
 * @return true, if any of the elements of document.forms[0].elements are
 *               different from those in oldElements;
 *         false, otherwise.
 */
function formElementsChanged(oldElements, indices) {
    for (var j = 0; j < indices.length; j++) {
        var state = extractElementState(document.forms[0].elements[indices[j]]);
        if (state != oldElements[j]) {
            return true;
        }
    }
    return false;
}

/**
 * Returns an Array that contains the form elements state.
 * @return an Array that contains the form elements state.
 */
function saveForm() {
    var indices = Array(document.forms[0].elements.length);
    for (var i = 0; i < document.forms[0].elements.length; i++) {
        indices[i] = i;
    }
    return saveFormElements(indices);
}

/**
 * Checks if a form changed.
 * @param oldForm the old form.
 * @return true if any of the elements of document.forms[0].elements[]
 */
function formChanged(oldForm) {
    var indices = Array(document.forms[0].elements.length);
    for (var i = 0; i < document.forms[0].elements.length; i++) {
        indices[i] = i;
    }
    return formElementsChanged(oldForm, indices);
}

function alertFormChanged(text) {
    alert(text);
}

function printList(list) {
    switch (list.length) {
        case 0: return "";
        case 1: return list[0];
        case 2: return list[0] + " and " + list[1];
        default:
            var output = "";
            for (var i = 0; i < list.length-1; i++) {
                output = output + list[i] + ", ";
            }
            return output + "and " + list[list.length-1];
    }
}

// String validation.

/**
 * Tests if the characters of a string are in a set.
 * @param s   the string whose characters should be tested.
 * @param set a string containing the set of characters.
 * @return true, if all the characters from s are in set;
 *         false, otherwise.
 */
function isStringCharsInSet(s, set) {
    for (var i = 0; i < s.length; i++) {
        if (set.indexOf(s.charAt(i)) == -1) {
            return false;
        }
    }
    return true;
}

/**
 * Gets a list of characters of a string that are not in a set.
 * @param s   the string whose characters should be tested.
 * @param set a string containing the set of characters.
 * @return a string containing the index and value of all
 *         the characters from s that are not in set.
 */
function getStringCharsNotInSet(s, set) {
    var badChars = new Array();
    for (var i = 0; i < s.length; i++) {
        if (set.indexOf(s.charAt(i)) == -1) {
            var bad = "" + (i+1) + ' (' + s.charAt(i) + ')';
            badChars.push(bad);
        }
    }
    return printList(badChars);
}

/**
 * Tests if the characters of a string are in a range.
 * @param s     the string whose characters should be tested.
 * @param start an ASCII integer representing the start of the range.
 * @param end   an ASCII integer representing the end of the range.
 * @return true, if all the characters from s have ASCII values between
 *               start and end;
 *         false, otherwise.
 */
function isStringCharsInRange(s, start, end) {
    for (var i = 0; i < s.length; i++) {
        var code = s.charCodeAt(i);
        if (code < start || code > end) {
            return false;
        }
    }
    return true;
}

/**
 * Gets a list of characters of a string that are not in a range.
 * @param s     the string whose characters should be tested.
 * @param start an ASCII integer representing the start of the range.
 * @param end   an ASCII integer representing the end of the range.
 * @return a string containing the index and value of all
 *         the characters from s who do not have ASCII values between
 *         start and end.
 */
function getStringCharsNotInRange(s, start, end) {
    var badChars = new Array();
    for (var i = 0; i < s.length; i++) {
        var code = s.charCodeAt(i);
        if (code < start || code > end) {
            var bad = "" + (i+1) + ' (' + s.charAt(i) + ')';
            badChars.push(bad);
        }
    }
    return printList(badChars);
}

/**
 * Tests if a string is valid in Hostapd.
 * @param s     the string whose characters should be tested.
 * @return true, if s does not include an ASCII 10 or 0;
 *         false, otherwise.
 */
function isHostapdString(s) {
    for (var i = 0; i < s.length; i++) {
        var code = s.charCodeAt(i);
        if (code == 0 || code == 10) {
            return false;
        }
    }
    return true;
}

/**
 * Gets a text reason the string is invalid in Hostapd.
 * @param s     the string that is invalid in Hostapd.
 * @return a string containing a text reason.
 */
function getInvalidHostapdStringReason(s) {
    var nulls = 0;
    var newlines = 0;

    for (var i = 0; i < s.length; i++) {
        var code = s.charCodeAt(i);
        if (code == 0) {
            nulls++;
        } else if (code == 10) {
            newlines++;
        }
    }

    var msg = "";
    if (nulls > 0 || newlines > 0) {
        msg = msg + "The string contains ";
    }
    if (nulls == 1) {
        msg = msg + "a null";
    } else if (nulls > 1) {
        msg = msg + "nulls";
    }
    if (nulls > 0 && newlines > 0) {
        msg = msg + " and ";
    }
    if (newlines == 1) {
        msg = msg + "a newline";
    } else if (newlines > 1) {
        msg = msg + "newlines";
    }
    if (nulls > 0 || newlines > 0) {
        msg = msg + ".";
    }
   
    return msg;
}

/**
 * Tests if the length of a string is in a range.
 * @param s   the string whose length should be taken.
 * @param min an integer giving the minimum length.
 * @param max an integer giving the maximum length.
 * @return true, if s.length is between min and max inclusive;
 *         false, otherwise.
 */
function isStringLengthInRange(s, min, max) {
    if (s.length < min || s.length > max) {
        return false;
    } else {
        return true;
    }
}

/**
 * Gets a reason string when the length of a string is not in a range.
 * @param s   the string whose length should be taken.
 * @param min an integer giving the minimum length.
 * @param max an integer giving the maximum length.
 * @return if s.length is between min and max inclusive, an empty string;
 *         otherwise, a string explaining that it should be.
 */
function getStringLengthNotInRangeReason(s, min, max) {
    if (s.length < min || s.length > max) {
        return 'The string is ' 
            + s.length
            + ' characters long, but must be between '
            + min
            + ' and '
            + max
            + ' characters long.';
     } else {
        return "";
     }
}

/**
 * Tests if a string is a specified length.
 * @param s   the string whose length should be taken.
 * @param len the specified length.
 * @return true, if s.length is equal to len;
 *         false, otherwise.
 */
function isStringLength(s, len) {
    return s.length == len;
}

/**
 * Gets a reason string when a string is not a specified length.
 * @param s   the string whose length should be taken.
 * @param len the specified length.
 * @return if s.length is equal to len, an empty string;
 *         otherwise, a string explaining that it should be.
 */
function getStringNotLengthReason(s, len) {
    if (s.length != len) {
        return 'The string is ' 
            + s.length
            + ' characters long, but must be '
            + len
            + ' characters long.';
     } else {
        return "";
     }
}

// Number validation

function isIntValid(i) {
    var number;
    number = parseInt(i);
    if (isNaN(number) || number != i) {
        return false;
    } else {
        return true;
    }
}

function getIntNotValidReason(i) {
    var number;
    number = parseInt(i);
    if (isNaN(number)) {
        return 'The value is not a valid integer.';
    } else if (number != i) {
        return 'The value contains a non-numerical character.';
    } else {
        return '';
    }
}

function isIntInRange(i, min, max) {
    if (! isIntValid(i) || i < min || i > max) {
        return false;
    } else {
        return true;
    }
}

function getIntNotInRangeReason(i, min, max) {
    if (! isIntValid(i)) {
        return getIntNotValidReason(i);
    } else if (i < min) {
        return 'The value must be at least ' + min + '.';
    } else if (i > max) {
        return 'The value must be at most ' + max + '.';
    } else {
        return '';
    }
}

function isIntEven(i) {
    if (! isIntValid(i) || i % 2 != 0) {
        return false;
    } else {
        return true;
    }
}

function getIntNotEvenReason(i) {
    if (! isIntValid(i)) {
        return getIntNotValidReason(i);
    } else if (i % 2 != 0) {
        return 'The value is not even.';
    } else {
        return '';
    }
}

function isFloatValid(f) {
    var number;
    number = parseFloat(f);
    if (isNaN(number) || (number.toFixed() != f && number.toFixed(1) != f)) {
        return false;
    } else {
        return true;
    }
}

function getFloatNotValidReason(f) {
    var number;
    number = parseFloat(f);
    if (isNaN(number)) {
        return 'The value is not a valid floating point number.';
    } else if (number.toFixed() != f && number.toFixed(1) != f) {
        return 'The value contains a non-numerical character.';
    } else {
        return '';

    }
}

function isFloatInRange(f, min, max) {
    if (! isFloatValid(f) || f < min || f > max) {
        return false;
    } else {
        return true;
    }
}

function getFloatNotInRangeReason(f, min, max) {
    if (! isFloatValid(f)) {
        return getFloatNotValidReason(f);
    } else if (f < min) {
        return 'The value must be at least ' + min + '.';
    } else if (f > max) {
        return 'The value must be at most ' + max + '.';
    } else {
        return '';
    }
}

function isValidWepKey(key, bytes, ascii) {
    var len = wepKeyCharsRequired(bytes, ascii);
    if (! isStringLength(key, len)) {
        return false;
    }

    if (ascii) {
        return isStringCharsInRange(key, 32, 126);
    } else {
        return isStringCharsInSet(key, "0123456789abcdefABCDEF");
    }
}

function isValidWepKeyOrBlank(key, bytes, ascii) {
    return key.length == 0 || isValidWepKey(key, bytes, ascii);
}

function getInvalidWepKeyReason(key, bytes, ascii) {
    var msg = "";

    var len = wepKeyCharsRequired(bytes, ascii);
    var lengthBad = false;
    if (! isStringLength(key, len)) {
        msg = msg + getStringNotLengthReason(key, len);
        lengthBad = true;
    }

    if (ascii) {
        if (! isStringCharsInRange(key, 0, 255)) {
            if (lengthBad) {
                msg = msg + ' ';
            }
            msg = msg
                + 'The following characters are not allowed: '
                + getStringCharsNotInRange(key, 0, 255)
                + '.';
        }
    } else {
        if (! isStringCharsInSet(key, "0123456789abcdefABCDEF")) {
            if (lengthBad) {
                msg = msg + ' ';
            }
            msg = msg
                + 'The following characters are not allowed: '
                + getStringCharsNotInSet(key, "0123456789abcdefABCDEF")
                + '.';
        }
    }

    return msg;
}

function wepKeyCharsRequired(bytes, ascii) {
    if (ascii) {
        return bytes;
    } else { // hex
        return bytes * 2;
    }
}

function isMacValid(mac) {
    if (mac.length != 17) {
        return false;
    }

    for (var i = 0; i < 6; i++) {
        var octet = mac.substr(i*3, 2);
        
        if (! isStringCharsInSet(octet, "0123456789abcdefABCDEF")) {
            return false;
        }
    }

    for (var i = 0; i < 5; i++) {
        var colon = mac.charAt(2+i*3);

        if (colon != ":") {
            return false;
        }
    }

    return true;
}

function getInvalidMacReason(mac) {
    if (mac.length != 17) {
        return "A MAC address must have six colon-separated octets in "
               + "hexadecimal form.";
    }

    for (var i = 0; i < 6; i++) {
        var octet = mac.substr(i*3, 2);
        
        if (! isStringCharsInSet(octet, "0123456789abcdefABCDEF")) {
            return "Octet " + (i+1) + " (" + octet + ") contains invalid "
                   + "hexadecimal characters.";
        }
    }

    for (var i = 0; i < 5; i++) {
        var colon = mac.charAt(2+i*3);

        if (colon != ":") {
            return "Separator " + (i+1) + " (" + colon + ") is not a colon.";
        }
    }

    return "";
}

function isVlanIdValid(vlanId) {
    return isIntInRange(vlanId, 1, 4094);
}

function getInvalidVlanIdReason(vlanId) {
    return getIntNotInRangeReason(vlanId, 1, 4094);
}

function isDomainLabelValid(label) {
    if (label.length > 63 || label.length < 1) {
        return false;
    }

    switch (label.length) {
    case 1:
        return isStringCharsInSet(label.substr(0, 1),
                                  LETTER_SET);
        break;
    case 2:
        return isStringCharsInSet(label.substr(0, 1),
                                  LETTER_SET)
               && isStringCharsInSet(label.substr(label.length-1, 1),
                                     LETTER_DIGIT_SET);
        break;
    default:
        return isStringCharsInSet(label.substr(0, 1),
                                  LETTER_SET)
               && isStringCharsInSet(label.substr(1, label.length-2),
                                     LETTER_DIGIT_HYPHEN_SET)
               && isStringCharsInSet(label.substr(label.length-1, 1),
                                     LETTER_DIGIT_SET);
    }
}

function getInvalidDomainLabelReason(label) {
    if (label.length > 63) {
        return "The value must be at most 63 characters long.";
    }

    if (label.length < 1) {
        return "The value must be at least 1 character long.";
    }

    switch (label.length) {
    case 1:
        if (! isStringCharsInSet(label.substr(0,1), LETTER_SET)) {
            return "A single-character value must be a letter.";
        }
        break;
    case 2:
        if (! isStringCharsInSet(label.substr(0,1), LETTER_SET)) {
           return "The first character must be a letter.";
        } else if (! isStringCharsInSet(label.substr(label.length-1, 1),
                                        LETTER_DIGIT_SET)) {
            return "The last character must be a letter or digit.";
        }
        break;
    default:
        if (! isStringCharsInSet(label.substr(0,1), LETTER_SET)) {
           return "The first character must be a letter.";
        } else if (! isStringCharsInSet(label.substr(1, label.length-2),
                                        LETTER_DIGIT_HYPHEN_SET)) {
           return "The middle characters must be letters, digits, or hyphens.";
        } else if (! isStringCharsInSet(label.substr(label.length-1, 1),
                                        LETTER_DIGIT_SET)) {
            return "The last character must be a letter or digit.";
        }
        break;
    }
    return "";    
}

// cookies ...
function createCookie(name,value,days) {
  if (days) {
    var date = new Date();
    date.setTime(date.getTime()+(days*24*60*60*1000));
    var expires = "; expires="+date.toGMTString();
  }
  else expires = "";
  document.cookie = name+"="+value+expires+"; path=/";
}

function readCookie(name) {
  var nameEQ = name + "=";
  var ca = document.cookie.split(';');
  for(var i=0;i < ca.length;i++) {
    var c = ca[i];
    while (c.charAt(0)==' ') c = c.substring(1,c.length);
    if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
  }
  return null;
}


// loading event management
var onLoadListeners;

function addOnLoadHandler(handler) {
	if( !onLoadListeners ) onLoadListeners = new Array();
	if( window.onload && window.onload != onLoadHandler ) {
		onLoadListeners[onLoadListeners.length] = window.onload;
	}		
	window.onload = onLoadHandler;
	onLoadListeners[onLoadListeners.length] = handler;
}

function onLoadHandler()
{
  if( onLoadListeners ) {
	for( var i = 0; i < onLoadListeners.length; i++) {
	       onLoadListeners[i]();
	}
  }
}
var onUnLoadListeners;
function addOnUnLoadHandler(handler) {
	if( !onUnLoadListeners ) onUnLoadListeners = new Array();
	if( window.onunload && window.onunload != onUnLoadListeners ) {
		onUnLoadListeners[onUnLoadListeners.length] = window.onunload;
	}		
	window.onunload = onLoadHandler;
	onUnLoadListeners[onUnLoadListeners.length] = handler;
}
function onUnLoadHandler()
{
	for( var i = 0; i < onLoadListeners.length; i++) {
		onLoadListeners[i]();
	}
}


