/* Copyright 2004 Instant802 Networks, Inc. All Rights Reserved. */

/**
 * The purpose of this module is to allow template authors to apply behaviors to
 * html elements by simply specifying a class.
 * We leverage the fact that elements may specify more than one class: ex. "<input type='text' class='foo bar myclass'>"
 */

/**
 * determines if given object is of the class specified by the given name.
 */
function isClass(obj, name) {
	var classes = obj.className.split( "/\\s+/gi" );
	for( i = 0; i < classes.length; i++ ) {
		if( classes[i] == name )
			return true;
	}
			
	return false;
}

/**** input checkbox class. ***/

/** Toggle all other checkboxs in the containing form. */
function onclick_toggleAll_checkbox() { //v2.0
	var elms = this.form.elements;
	for(var i = 0; i < elms.length; i++) {
		var elm = elms[i];
		if(elm.type == "checkbox") {
			if(!isClass(elm, "sync_all-checkbox")) { // don't set self
				elm.checked = this.checked;
			}
			if(isClass(elm, "row_select-checkbox")) {
				elm.onclick();
			}
		}
	}
}

/** change background of containing table row. */
function onclick_selectRow_checkbox() {
	var parent = this.parentNode;
	var row;
	while(parent) {
			if(parent.tagName == "TR") {
				row = parent;
				break;
			}
			parent = parent.parentNode;
	}
	if(row) {
		if(this.checked) {
			if("undefined" == (typeof row.unselectedBgColor)) { // if prop is not defined, save background color.
				row.unselectedBgColor = row.style && row.style.backgroundColor ? row.style.backgroundColor : "";
			}
			row.style.backgroundColor = "#bebeef";
		}
		else {
			if(row.unselectedBgColor || row.unselectedBgColor == "") {
				row.style.backgroundColor = row.unselectedBgColor;
			}
		}
	}
}

function textInputKeyPressHandler( keyEvent ) {
	if( this.filter ) {
		var e = window.event ? window.event : keyEvent;
		var charCode = (e.charCode) ? e.charCode : ((e.keyCode) ? e.keyCode : ((e.which) ? e.which : null));
		var ret = this.filter( String.fromCharCode(charCode) );
		if(e.returnValue) e.returnValue = ret;
		if(!ret && e.preventDefault) e.preventDefault();
		return ret;
	}
	return true;
}


/** 
 * look for all "widgets" (i.e. form elements as of this writing) and apply behaviors on those elements that
 * declare certain class names.  note: elements may specify more than one class: ex. "<input type='text' class='foo bar myclass'>"
 * tbd: need more generalized approach.  use naming convention <event>_<class>_<input-type>?
 * methods should append not replace.
 * formalize the association of class name with behaviors.
 */
function initializeWidgets() {
	var formObjs = document.forms;
	for(var i = 0; i < formObjs.length; i++) {
		var elementObjs = formObjs[i].elements;
		for(var j = 0; j < elementObjs.length; j++) {
			var elm = elementObjs[j];
			if(isClass(elm, "row_select-checkbox")) {
				elm.onclick = onclick_selectRow_checkbox;
			}
			if(elm.type == "checkbox") {
				if(isClass(elm, "sync_all-checkbox")) {
					elm.onclick = onclick_toggleAll_checkbox;
				}
			}
			if(elm.type == "text") {
				if(isClass(elm, "alphanumeric")) {
					elm.filter = isAlphaNumeric;
					elm.onkeypress = textInputKeyPressHandler;
				}
				if(isClass(elm, "numeric")) {
					elm.filter = isNumeric;
					elm.onkeypress = textInputKeyPressHandler;
				}
				if(isClass(elm, "alphabetic")) {
					elm.filter = isAlphabetic;
					elm.onkeypress = textInputKeyPressHandler;
				}
			}
		}
	}
}

addOnLoadHandler(initializeWidgets);
