/*
 * Created on Sep 17, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.LSQL;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.deviceconfiguration.*;
import com.instant802.cldc.deviceconfiguration.ConfigObject;
import com.instant802.cldc.deviceconfiguration.Schema;
import com.instant802.cldc.net.HTTPClient;
import com.instant802.cldc.net.HTTPRequest;
import com.instant802.cldc.net.HTTPResponse;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class LSQLClient implements MutableConfiguration {
	
	public static final String ORIGIN = "orchestrator";
	private static final String UNSAFE = "$&+,/:;=?@ '\"<>#%{}|\\~^[]`";
	private static final int RETRIES = 5;
	private static final String TYPE = "LSQLClient";
	private final Schema schema;
	
	public LSQLClient( Schema schema ) {
		this.schema = schema;
	}
	
	/**
	 * return the current full configuration read from APConfD as
	 * a string.  Retry on communications problems.
	 */
	public byte[] configAsDocument() throws IOException, MalformedHTTPMessageException, SocketException {
		HTTPResponse response;
		int retries = 0;
		while( retries < RETRIES ) {
			retries ++;
			try {
				response = doQuery("<?xml%20version=\"1.0\"?><get_conf/>");
				if( response.getStatusCode() != 200 ) {
					if( retries > RETRIES ) {
						throw new IOException("Failed to get configuration document - http status was: "+response.getStatusLine() );
					} else {
						warning("got http status "+response.getStatusLine()+" when trying to get config");
					}
				}
				return response.getBody();
			} catch (IOException e) {
				if( retries > RETRIES ) {
					throw ( e );
				} else {
					warning( e );
				}
			} catch (MalformedHTTPMessageException e) {
				if( retries > RETRIES ) {
					throw e;
				} else {
					warning( e );
				}
			}
		}
		throw new IOException("failed to read configuration after "+retries+" retries.");
	}
	
	private void warning( Exception e ) {
		Log.warning( TYPE, e );
	}
	
	private void warning(String string) {
		Log.warning( TYPE, string );
	}

	public LSQLResultSet query( String query ) throws LSQLException, XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		HTTPResponse response = getResponse( "<lsql>" + query + "</lsql>" );
		return new LSQLResultSet( schema, response );
	}
	
	private LSQLResultSet update( String query ) throws XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		int retries = 0;
		while( retries < RETRIES ) {
			retries ++;
			HTTPResponse response;
			try {
				if (Log.DEBUG) debug("try "+retries+" to perform: "+ query);
				response = getResponse(query);
				LSQLResultSet results = new LSQLResultSet( schema, response );
				return results;
			} catch (IOException e) {
				if( retries > RETRIES ) {
					throw e;
				} else {
					warning( e );
				}
			} catch (MalformedHTTPMessageException e) {
				if( retries > RETRIES ) {
					throw e;
				} else {
					warning( e );
				}
			}
		}
		throw new IOException("failed to perform update "+query+" after "+retries+" retries.");
	}

	private void debug(String string) {
		Log.debug("LSQLClient", string );
	}

	private HTTPResponse getResponse(String query) throws IOException, MalformedHTTPMessageException, SocketException {
		HTTPResponse response;
			response = doQuery( urlEscape( query ).toString() );
		if( response.getStatusCode() != 200 ) {
			throw new IOException("unexpected result from webserver: "+response.getStatusLine() );
		}
		return response;
	}
	
	private synchronized static HTTPResponse doQuery(String query)
		throws IOException, MalformedHTTPMessageException, SocketException {
		HTTPClient client = new HTTPClient();
		HTTPRequest request = new HTTPRequest("/cgi-bin/notify.cgi?object=lsql&lsql="+query );
		request.setHost("127.0.0.1");
		HTTPResponse response = client.doRequest( request );
		return response;
	}

	private StringBuffer urlEscape( String query ) {
		StringBuffer escaped = new StringBuffer();
		for( int i = 0; i < query.length(); i++ ) {
			char c = query.charAt( i );
			if( ! isUnsafe( c ) ) {
				escaped.append( c );
			} else {
				escaped.append( escapeChar( c ) );
			}
		}
		return escaped;
	}

	private String escapeChar(char c) {
		return "%" + Integer.toString( c, 16 );
	}
	
	private boolean isUnsafe( char c ) {
		if( c < 32) return true;
		if( c > 126 ) return true;
		return( UNSAFE.indexOf( c ) > -1);
	}

	public void updateObject(FieldChanges2 changes) throws LSQLException, APConfException {
		update( new LSQLUpdate( changes ) );
	}

	public void addObject(ConfigObject object) throws LSQLException, APConfException {
		update( new LSQLInsert( object ) );
	}

	void update(LSQLQuery query) throws LSQLException, APConfException {
		try {
			StringBuffer buffer = query.asXMLDocument();			
			LSQLResultSet result = update( buffer.toString() );
			if( result.resultCode() != 0) {
				throw new LSQLException( "update failed with result code "+result.resultCode()+" message: "+result.error() );
			}
		} catch (XMLRepresentationException e) {
			throw new APConfException( e );
		} catch (IOException e) {
			throw new APConfException( e );
		} catch (SocketException e) {
			throw new APConfException( e );
		} catch (MalformedHTTPMessageException e) {
			throw new APConfException( e );
		}
		if (Log.DEBUG) debug("done with update");
	}

	public void deleteObject(ConfigObject object) throws LSQLException, APConfException {
		update( new LSQLDelete( object ) );
	}

	public void setLocalRunningConfig() throws LSQLException, APConfException {
		if (Log.DEBUG) debug("setLocalRunningConfig");
		update( new LSQLSetLocalRunningConfig() );
	}
	
	public void setRunningConfig() throws LSQLException, APConfException {
		if (Log.DEBUG) debug("setRunningConfig");
		update( new LSQLSetRunningConfig() );
	}	
}
