/*
 * Created on Oct 4, 2003
 * written by robin
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.LSQL;

import java.util.Enumeration;

import com.instant802.cldc.deviceconfiguration.ConfigObject;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 */
public abstract class LSQLObjectQuery extends LSQLQuery implements LSQLStatement {

	private static final String LSQL = "lsql";
	public static final String INSERT = "insert";
	public static final String UPDATE = "update";
	public static final String DELETE = "delete";
	public static final String WHERE = "where";
	public static final String EQ = "eq";
	public static final String SCALAR = "scalar";
	public static final String FIELD = "field";
	protected final ConfigObject object;
	
	LSQLObjectQuery( ConfigObject object ) {
		this.object = object;
	}
	
	public StringBuffer asXMLDocument() {
		StringBuffer document = new StringBuffer();
		appendDocumentStart( document );
		openTag( document, LSQL );
		appendQuery( document );
		closeTag( document, LSQL );
		return document;
	}
	
	private void debug( String debug ) {
		Log.debug("LSQLObjectQuery", debug );
	}

	protected void closeType(StringBuffer document) {
		closeTag( document, object.typeName() );
	}

	protected void openType(StringBuffer document) {
		if( object.hasNameAttribute() ) {
			openNameTag( document, object.typeName(), object.getNameAttribute() );
		} else {
			openTag( document, object.typeName() );
		}
	}

	protected void openOriginTag( StringBuffer document, String operation ) {
		document.append( "<" ).append( operation ).append(" origin=\"");
		document.append( LSQLClient.ORIGIN ).append("\">");
	}

	protected void appendPrimaryKey(StringBuffer document) {
		object.primaryKey().addToLSQLStatement( this, document, object );
	}

	public void appendWhere(StringBuffer document, String field, String value) {
		openTag( document, WHERE );
		openTag( document, EQ );
		openTag( document, FIELD );
		document.append( field );
		closeTag( document, FIELD );
		openTag( document, SCALAR );
		appendData( document, value );
		closeTag( document, SCALAR );
		closeTag( document, EQ );
		closeTag( document, WHERE );
	}

	protected void appendFields(StringBuffer document) {
		if (Log.DEBUG) debug("appending fields");
		Enumeration fieldNames = object.fieldNames();
		
		while( fieldNames.hasMoreElements() ) {
			String fieldName = (String) fieldNames.nextElement();
			if (Log.DEBUG) debug("appending field "+fieldName );
			if( object.writeField( fieldName ) ) {
				appendField(document, fieldName);
			}
		}
	}

	private void appendField(StringBuffer document, String fieldName) {
		openTag( document, fieldName );
		appendData( document, object.field( fieldName ) );
		closeTag( document, fieldName );
	}
}
