/*
 * Created on Oct 25, 2003
 * author: Robin Barooah
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.clustermembership;

import java.util.Enumeration;
import java.util.Hashtable;

import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.deviceconfiguration.ClusteredAP;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 */
public class MarkedForRemoval {

	private final MembershipList list;
	private final OrchestratorNode node;
	private final Hashtable marked = new Hashtable();

	MarkedForRemoval( MembershipList list, OrchestratorNode node ) {
		this.list = list;
		this.node = node;
	}

	/**
	 * delete the aps listed in this set from the configuration model.
	 */
	public void deleteFromConfig() throws LSQLException, APConfException {
		synchronized( list ) {
			Enumeration e = marked.elements();
			while( e.hasMoreElements() ) {
				ClusteredAP ap = (ClusteredAP) e.nextElement();
				list.removeMember( ap );
			}
		}
	}

	/**
	 * return true if the set of aps includes this node.
	 */
	public boolean includesThis() 
	{
		Log.info("checking for "+node.getNodeId()+" in set marked for removal");
		return marked.containsKey( node.getNodeId() );
	}

	/**
	 * add a clustered ap to the marked set.
	 */
	void add(ClusteredAP ap) {
		marked.put( ap.id(), ap );
	}

	public Enumeration elements() {
		return marked.elements();
	}

	public boolean notEmpty() {
		return marked.size() > 0;
	}
}
