/*
 * Created on Sep 28, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.coordination;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.clustermembership.MembershipAcceptedMessage;
import com.instant802.cldc.deviceconfiguration.Schema;
import com.instant802.cldc.messaging.Message;
import com.instant802.cldc.messaging.MessageClassifier;
import com.instant802.cldc.messaging.MessageNotUnderstoodException;
import com.instant802.cldc.net.Socket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.SocketTimeoutException;
import com.instant802.cldc.transfer.TransferBase;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.ThreadCounter;

/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ClientAgent extends TransferBase implements ControlledThread
{
	private static final String TYPE = "ClientAgent";
	private static final String DEFAULT_SENDER_ID = "newly connected client";
	private final CoordinatorServer server;
	private boolean done = false;
	private String senderID = DEFAULT_SENDER_ID;
	private final Thread thread = new Thread( this );
	private final int SHUTDOWN_TIMEOUT = 1000;
	/**
	 * @param msg
	 */
	
	public ClientAgent( CoordinatorServer server, Schema schema, Socket socket ) 
	{
		super( schema, socket );
		this.server = server;
		ThreadCounter.register( thread );
	}

	public void finalize() {
		ThreadCounter.deregister( thread );
	}

	public void run()
	{
		if (Log.DEBUG) debug("starting thread");
		try {
		while (!done)
		{
			if (Log.DEBUG) info("waiting for next message from "+senderID );
			try
			{
				XMLRecord msg;
				msg = (XMLRecord) receive();
				Message received = MessageClassifier.classifyMessage( this, msg );
				setSenderID( received.getSenderId() );
				MessageHolder holder = new MessageHolder( received );
				received = null;
				server.handleMessage( holder );
			} catch ( SocketTimeoutException e ) {
				// NOT AN ERROR
			}
		}
		} catch (SocketException e1) {
			if (Log.DEBUG) info("socket exception in agent");
			error( e1 );
		} catch (IOException e1) {
			error( e1 );			
		} catch (XMLRepresentationException e1) {
			error( e1 );
		}	catch (MessageNotUnderstoodException e) {
			Log.emergency( e );
	   	} catch (Exception e ) {
	   		Log.emergency( e );
	   	} catch (Throwable t ) {
	   		Log.emergency( t );
	   	}
	   	if (Log.DEBUG) info("exiting agent loop");
		server.dropAgent( this );
		close();
		if (Log.DEBUG) info("exiting agent run method");
	}
	
	private void info(String string) {
		Log.info( "ClientAgent", string );
	}

	private synchronized void setSenderID(String string) {
		senderID = string;
	}
	
	public synchronized String id() {
		return senderID;
	}

	public void error( Exception e ) {
		Log.error(TYPE ,  e );
	}

	public void debug( String  msg ) {
		Log.debug(TYPE , msg );
	}

	public void close() {
		signalToStop();
		super.close();
	}
	
	private void signalToStop() {
		if (Log.DEBUG) debug("setting done to true");
		done = true;
		if (Log.DEBUG) debug("setting timeout");
//		setTimeout( SHUTDOWN_TIMEOUT );
		if (Log.DEBUG) debug("done setting timeout - finished signalling to stop");
	}
	
	public void waitForStop() {
		if (Log.DEBUG) debug("signalling agent to stop");
		signalToStop();
//
// we can't safely do this join because we may be holding locks that are needed by the 
// thread with which we're joining.
//
//		try {
//			debug("joining thread to await completion (thread.alive ="+thread.isAlive() );
//			thread.join(); 
//		} catch (InterruptedException e) {
//			error( e );
//		}
	}

	
	
	public void send(Message received) throws SocketException {
		super.send( received );
		/* only after a membership accepted has been issued is the agent added to the 
		 * server to receive broadcasts.
		 * TODO: this whole transferagent/layer thing needs refactoring.
		 */
		if (Log.DEBUG) debug("sent "+received.typeName() );
		if( received.typeName().equals( MembershipAcceptedMessage.TYPE )) {
			if (Log.DEBUG) debug("adding agent to approved list");
			server.addAgent( this );
		}
	}

	public void send(StringBuffer message, String typeName ) throws SocketException {
		super.send( message );
		/* only after a membership accepted has been issued is the agent added to the 
		 * server to receive broadcasts.
		 * TODO: this whole transferagent/layer thing needs refactoring.
		 */
		if (Log.DEBUG) debug("sent "+typeName );
		if( typeName.equals( MembershipAcceptedMessage.TYPE )) {
			if (Log.DEBUG) debug("adding agent to approved list");
			server.addAgent( this );
		}
	}

	
	public void start() {
		if (Log.DEBUG) debug("starting agent");
		thread.start();
	}
}
