/*
 * Created on Sep 5, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.orchestrator;

import java.io.IOException;

import com.instant802.cldc.radius.Proxy;
import com.instant802.cldc.radius.RadiusServer;
import com.instant802.cldc.radius.session.AuthenticationHistory;
import com.instant802.cldc.ssl.InvalidCertificateException;
import com.instant802.cldc.ssl.InvalidPrivateKeyException;
import com.instant802.cldc.ssl.SSLSessionContext;
import com.instant802.cldc.Activation.Activator;
import com.instant802.cldc.Activation.ActivatorException;
import com.instant802.cldc.Activation.Service;
import com.instant802.cldc.Activation.ServiceException;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLClient;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.LSQL.LSQLMessageReceiver;
import com.instant802.cldc.SessionMonitor.AssociationModel;
import com.instant802.cldc.channelplanner.WirelessNeighborhoodModel;
import com.instant802.cldc.clustermastering.ClusterMasterAgent;
import com.instant802.cldc.clustermastering.ClusterMasterAgent2;
import com.instant802.cldc.clustermastering.MasterCriteria;
import com.instant802.cldc.clustermembership.MembershipAgent;
import com.instant802.cldc.coordination.CoordinationAgent;
import com.instant802.cldc.deviceconfiguration.ConfigObject;
import com.instant802.cldc.deviceconfiguration.ConfigurationModel;
import com.instant802.cldc.deviceconfiguration.DebugConfig;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.deviceconfiguration.SystemConfig;
import com.instant802.cldc.deviceconfiguration.ObjectChangeProcessor;
import com.instant802.cldc.deviceconfiguration.OrchestratorConfig;
import com.instant802.cldc.deviceconfiguration.Schema;
import com.instant802.cldc.discovery.DiscoveryEngine;
import com.instant802.cldc.messaging.Message;
import com.instant802.cldc.messaging.MessageReceiver;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.nodestatus.NodeStatusMonitor;
import com.instant802.cldc.nodestatus.NodeStatusPackage;
import com.instant802.cldc.nodestatus.NodeStatusRecord;
import com.instant802.cldc.synchronization.SynchronizationAgent;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.SyslogMessage;
import com.instant802.cldc.util.SyslogSender;

/**
 * @author robin
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class OrchestratorNode {

        private static final String CERTIFICATE_FILE = "/etc/cacert.pem";
        private static final String PRIVATE_KEY_FILE = "/etc/key.pem";
        public static final int COORDINATION_PORT = 39851; // this is TCP
        private static final int DISCOVERY_PORT = 39851; // this is UDP
        private static final int MEMBERSHIP_SIZE_LIMIT = 12;
        private static final int STATUS_RUNNING = 0;
        public static final int STATUS_RESTART = 1;
        
        private final Activator activator; 
        private final Proxy proxy;
        private final AdminInterface adminInterface;
        private final DiscoveryEngine discoveryEngine;
        private final ClusterMasterAgent clusterMasterAgent;
        private final MessageReceiver messageReceiver;
        private final MembershipAgent membershipAgent;
        private final ConfigurationModel configModel;
        private final CoordinationAgent coordinationAgent;
        private final SSLSessionContext sslSessionContext;
        private final SynchronizationAgent synchronizationAgent;
        private final Schema schema;
        private final LSQLClient lsqlClient;
        private final LSQLMessageReceiver lsqlReceiver;
        private final MasterCriteria masterCriteria;
        private final NodeStatusRecord nodeStatusRecord;
        private final ObjectChangeProcessor objectChangeProcessor;
        private int status = STATUS_RUNNING;
        private Boolean statusLock = new Boolean( true );
        
        public  OrchestratorNode() throws LSQLException, IOException, InvalidCertificateException, InvalidPrivateKeyException {
                Log.notice("creating node");
                adminInterface = new AdminInterface( "br0" );
                SyslogMessage.setHostname( adminInterface.ipAddress() );
                Log.notice("creating new orchestrator node with address: "+adminInterface.ipAddress()+ totalMemory() );
                sslSessionContext = new SSLSessionContext( SSLSessionContext.MODE_PEER , CERTIFICATE_FILE, PRIVATE_KEY_FILE );
                debug("created event manager");
                activator = new Activator( this );
                debug("created activator");
                nodeStatusRecord = NodeStatusMonitor.record( this );
                NodeStatusPackage.Install( activator );
                masterCriteria = new MasterCriteria( this );
                //TODO: these constructors are seriously sequence dependent,
				// in a way that's hard
                //TODO: to verify. This should be sorted out for program
				// correctness.
                schema = new Schema(); // must come before configuration model
                configModel = new ConfigurationModel( this );
                objectChangeProcessor = new ObjectChangeProcessor( this );
                discoveryEngine = new DiscoveryEngine( this );
                messageReceiver = new MessageReceiver( this );
                clusterMasterAgent = new ClusterMasterAgent2( this );
                membershipAgent = new MembershipAgent( this );
                lsqlClient = new LSQLClient( schema );
                coordinationAgent = new CoordinationAgent( this );        
                synchronizationAgent = new SynchronizationAgent( this );
                //SynchronizationPackage.Install( activator ); // comment this
				// out to remove the manual synchronization interface
                lsqlReceiver = new LSQLMessageReceiver( this, configModel );  // this
																			  // line
																			  // makes
																			  // configuration
																			  // updates
																			  // synchronize
																			  // automatically
                //lsqlReceiver = new LSQLMessageReceiver( this,
				// synchronizationAgent ); // this line makes configuration
				// updates happen only when requested
                proxy = new Proxy(1812, false, configModel );
                Log.notice("node created "+ totalMemory() );                
        }
 
        public RadiusServer getRadiusServer()
		{
        	return proxy.getServer();
        }
        
        public ObjectChangeProcessor objectChangeProcessor() {
        	return this.objectChangeProcessor;
        }
        
        public NodeStatusRecord nodeStatusRecord() {
        	return nodeStatusRecord;
        }

	private void setupModelNumber() {
		try {
			SystemConfig config = configModel.getHostConfig();
			if (config.model() != ConfigObject.UNDEFINED_FIELD) {
				Message.setModelNumber(config.model());
				Log.info("model number of device is: " + config.model());
			} else {
				Log.info("model number in host object is: "+config.model()+" so not setting" );
			}
		} catch (ElementNotFoundException e) {
			Log.debug("device model number not set in config - using default");
		}
	}
        
        private void setupDebug() throws ElementNotFoundException {
                DebugConfig config = configModel.getDebugConfig();
                if( config.lostHost() != ConfigObject.UNDEFINED_FIELD ) {
                        SyslogSender.setServer( config.lostHost() );
                        Log.setRemote();
                }
                Log.setFilterLevel( config.level() );
        }

        private String totalMemory() {
                return " total memory is: "+Runtime.getRuntime().totalMemory()+" bytes." ;
        }

        /**
		 * @param string
		 */
        private void debug(String string) {
                Log.debug("OrchestratorNode", string );
        }

        public Activator activator() {
                return activator;
        }

         public String getNodeId() {
                 final String id = adminInterface.macAddress();
                 debug("returning node id: "+id);
                 return id;
         }
        
        public void broadcastMessage(Message message) throws SocketException {
                byte[] bytes = message.asXMLStringBufferDocument().getBytes();
                adminInterface.broadcast( bytes, DISCOVERY_PORT );
                Log.info("broadcast message: "+ message );
        }
                
        public AdminInterface adminInterface() {
                return adminInterface;
        }
        
        private final void setupPrecedence() {
        	try{
        		OrchestratorConfig config = configModel.getOrchestratorConfig();
        		
        		if( config.configRestored() )
        			masterCriteria.setBecomeMaster();
    
        		lsqlReceiver.clearConfigRestored();
        	} catch ( ElementNotFoundException e ) {
        		Log.error( e );
        	} catch ( LSQLException e ) {
        		Log.error( e );
        	} catch ( APConfException e ) {
        		Log.error( e );
        	}
        }
        
        public void start() throws SocketException, LSQLException, XMLRepresentationException, IOException, MalformedHTTPMessageException, ServiceException {
                Log.notice("starting node");
                configModel.populate();
                /* now the mastercriteria can check to see whether
                 * we should apply precedence to this node.
                 */
                setupPrecedence();
                service("channel-planner"); // To start the CPthread ...                
                try{
                        setupDebug();
                } catch ( ElementNotFoundException e ) {
                        Log.error( e );
                }
                setupModelNumber();
                debug("About to start Proxy server");
                new Thread( proxy ).start();
                messageReceiver.start();
                coordinationAgent.start();
                lsqlReceiver.start();
                activator.start();
                Log.notice("node started");
        }

        private void stopThreads() {
        	info("waiting for lsqlReceiver to stop");
        	lsqlReceiver.waitForStop();
        	info("waiting for messageReceiver to stop");
        	messageReceiver.waitForStop();
        	info("waiting for coordinationAgent to stop");
        	coordinationAgent.waitForStop();
        	try {
        		info("waiting for activator to stop");
				activator.waitForStop();
			} catch (ActivatorException e) {
				Log.error( e );
			}
			info("waiting for proxy to stop");
        	proxy.waitForStop();
        }
        
        private void info(String string) {
        	Log.info("OrchestratorNode", string );
		}

		/**
		 * @param string
		 * @return
		 */
        public Service service(String name) throws ServiceException {
                return activator.service( name );
        }

        /**
		 * @return
		 */
        public MembershipAgent membershipAgent() {
                return membershipAgent;
        }

        public AssociationModel associations() {
                return new AssociationModel("dummy radio mac");
        }

        public WirelessNeighborhoodModel wirelessneighbors() {
        		return new WirelessNeighborhoodModel(this);
        }
        
        public AuthenticationHistory authHistory() {
                return proxy.getAuthenticationHistory();
        }        
        
        public DiscoveryEngine discoveryEngine() {
                return discoveryEngine;
        }

        public ClusterMasterAgent clusterMasterAgent() {
                return clusterMasterAgent;
        }


        public MessageReceiver messageReceiver() {
                return messageReceiver;
        }

        
        public int getMessagePort() {
                return DISCOVERY_PORT;
        }

        public void exitStandalone() {
                try{
                Log.notice("Node "+getNodeId()+" at "+adminInterface.ipAddress()+" is exiting standalone mode now.");
                debug("starting cluster master agent");
                clusterMasterAgent.activate();
                debug("reactivating coordination agent");
                coordinationAgent.activate();
                Log.notice("Node "+getNodeId()+" at "+adminInterface.ipAddress()+" clustering restarted.");
                } catch (Exception e ) {
                        Log.critical( e );
                        Log.critical("failed to exit standalone mode");
                }
        }

        public void enterStandalone() {
                Log.notice("Node "+getNodeId()+" at "+adminInterface.ipAddress()+" going into standalone mode now.");
                // TODO: implement standalone mode logic
                
                clusterMasterAgent.deactivate();                
                coordinationAgent().deactivate();
                
                OrchestratorConfig config;
                try {
                        config = configModel.getOrchestratorConfig();
                        config.setClusterable( false );
                        config.setClusterID( MembershipAgent.NON_MEMBER );
                        masterCriteria().reset();
                        membershipAgent.dropClusteredAPs();
                        configModel.writeToAPConf();
                } catch (ElementNotFoundException e) {
                        Log.emergency( e );
                } catch (XMLRepresentationException e) {
                        Log.emergency( e );
                } catch (IOException e) {
                        Log.emergency( e );
                } catch (MalformedHTTPMessageException e) {
                        Log.emergency( e );
                } catch (LSQLException e) {
                		handleLSQLException( e );
                } catch (APConfException e) {
                	Log.emergency( e );
                } catch (SocketException e) {
                	Log.emergency( e );
				}
        }

        public void handleLSQLException( LSQLException e ) {
        	Log.debug("handling lsql exception");
//        	nodeStatusRecord().setClusterFailureStatus( e.getMessage() );
        	Log.emergency( e );
        	try {
        		Log.debug("repopulating configuration model");
				configurationModel().populate();
				Log.debug("done repopulating configuration model");
			} catch (XMLRepresentationException e1) {
				Log.emergency( e1 );
			} catch (IOException e1) {
				Log.emergency( e1 );
			} catch (MalformedHTTPMessageException e1) {
				Log.emergency( e1 );
			} catch (SocketException e1) {
				Log.emergency( e1 );
			}
			Log.debug("done handling LSQL exception");
        }
        
        public int getMembershipSizeLimit() {
                return MEMBERSHIP_SIZE_LIMIT;
        }

        public ConfigurationModel configurationModel() {
                return configModel;
        }

        public SSLSessionContext sslSessionContext() {
                return sslSessionContext;
        }

        public int getCoordinationPort() {
                return COORDINATION_PORT;
        }

        public SynchronizationAgent synchronizationAgent() {
                return synchronizationAgent;
        }


        public Schema schema() {
                return schema;
        }

        public LSQLClient lsqlClient() {
                return lsqlClient;
        }
        
        public CoordinationAgent coordinationAgent() {
                return coordinationAgent;
        }
        
        public MasterCriteria masterCriteria() {
                return masterCriteria;
        }
        
        public LSQLMessageReceiver lsqlMessageReceiver() {
        	return lsqlReceiver;
        }

		public void restart() {
			adminInterface.shutdown();
			Runtime.getRuntime().exit( 0 );
//			stopThreads();
//			Log.debug("threads stopped");
//			setStatus( STATUS_RESTART );
//			Log.debug("status set");
		}
		

		public void join() {
			synchronized( statusLock ) {
				if( status != STATUS_RUNNING ) {
					return;
				}
				try {
					statusLock.wait();
				} catch (InterruptedException e) {
					Log.error( e );
				}
			}
		}

		private void setStatus( int status ) {
			synchronized( statusLock ) {
				this.status = status;
				statusLock.notifyAll();
			}
		}
		
		public int exitStatus() {
			return status;
		}
}
