/*
 * Created on Jan 23, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package com.instant802.cldc.qdxml;

import java.io.*;
import java.util.Vector;

import com.instant802.cldc.util.Log;

/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class QDParser
{
	private int[] modes = new int[128];
	private int modei = 0;

	private final int popMode()
	{
		if (modei > 0)
			return modes[--modei];
		else
			return PRE;
	}

	private final void pushMode(int m)
	{
		if (modei < 128)
		{
			modes[modei++] = m;
		}
	}
	private final int TEXT = 1,
		ENTITY = 2,
		OPEN_TAG = 3,
		CLOSE_TAG = 4,
		START_TAG = 5,
		ATTRIBUTE_LVALUE = 6,
		ATTRIBUTE_EQUAL = 9,
		ATTRIBUTE_RVALUE = 10,
		QUOTE = 7,
		IN_TAG = 8,
		SINGLE_TAG = 12,
		COMMENT = 13,
		DONE = 11,
		DOCTYPE = 14,
		PRE = 15,
		CDATA = 16;

	public final void parse(DocHandler doc, byte[] chars)
		throws Exception
	{
		
		int depth = 0;
		int mode = PRE;
		int c = 0;
		int quotec = '"';
		depth = 0;
		final StringBuffer sb = new StringBuffer(256);
		final StringBuffer etag = new StringBuffer(16);
		String tagName = null;
		String lvalue = null;
		String rvalue = null;
		Vector attrs = null;
		doc.startDocument();
		boolean eol = false;
		boolean lasttagisopen = false;
		int len = chars.length;
		for (int i = 0; i < len; ++i)
		{
			c = chars[i];

			// We need to map \r, \r\n, and \n to \n
			// See XML spec section 2.11
			//			if (c == '\n' && eol)
			//			{
			//				eol = false;
			//				continue;
			//			} else if (eol)
			//			{
			//				eol = false;
			//			} else if (c == '\r')
			//			{
			//				eol = true;
			//				c = '\n';
			//			}

			switch (mode)
			{
				case DONE :
					doc.endDocument();
					return;

					// We are between tags collecting text.
				case TEXT :
					if (c == '<')
					{
						pushMode(mode);
						mode = START_TAG;
						if (chars[i+1] == '/' && lasttagisopen && sb.length() > 0)   // If next tag is an open tag chuck the text
								doc.text(sb.toString());
						sb.setLength(0);
					} else if (c == '&')
					{
						pushMode(mode);
						mode = ENTITY;
						etag.setLength(0);
					} else
						sb.append((char) c);
					break;

					// we are processing a closing tag: e.g. </foo>
				case CLOSE_TAG :
					if (c == '>')
					{
						mode = popMode();
						tagName = sb.toString();
						sb.setLength(0);
						depth--;
						if (depth == 0)
							mode = DONE;
						doc.endElement(tagName);
					} else
					{
						sb.append((char) c);
					}
					break;

					// we are processing CDATA
				case CDATA :
					if (c == '>' && sb.toString().endsWith("]]"))
					{
						sb.setLength(sb.length() - 2);
						doc.text(sb.toString());
						sb.setLength(0);
						mode = popMode();
					} else
						sb.append((char) c);
					break;

					// we are processing a comment. We are inside
					// the <!-- .... --> looking for the -->.
				case COMMENT :
					if (c == '>' && sb.toString().endsWith("--"))
					{
						sb.setLength(0);
						mode = popMode();
					} else
						sb.append((char) c);
					break;

					// We are outside the root tag element
				case PRE :
					if (c == '<')
					{
						pushMode(TEXT);
						mode = START_TAG;
					}
					break;

					// We are inside one of these <? ... ?>
					// or one of these <!DOCTYPE ... >
				case DOCTYPE :
					if (c == '>')
					{
						mode = popMode();
						if (mode == TEXT)
							mode = PRE;
					}
					break;

					// we have just seen a < and
					// are wondering what we are looking at
					// <foo>, </foo>, <!-- ... --->, etc.
				case START_TAG :
					mode = popMode();
					if (c == '/')
					{
						pushMode(mode);
						mode = CLOSE_TAG;
						lasttagisopen = false;						
					} else if (c == '?')
					{
						mode = DOCTYPE;
					} else
					{
						pushMode(mode);
						mode = OPEN_TAG;
						tagName = null;
						attrs = new Vector();
						sb.append((char) c);
						lasttagisopen = true;						
					}
					break;

					// we are processing an entity, e.g. &lt;, &#187;, etc.
				case ENTITY :
					if (c == ';')
					{
						mode = popMode();
						String cent = etag.toString();
						etag.setLength(0);
						if (cent.equals("lt"))
							sb.append('<');
						else if (cent.equals("gt"))
							sb.append('>');
						else if (cent.equals("amp"))
							sb.append('&');
						else if (cent.equals("quot"))
							sb.append('"');
						else if (cent.equals("apos"))
							sb.append('\'');
						// Could parse hex entities if we wanted to
						//else if(cent.startsWith("#x"))
						//sb.append((char)Integer.parseInt(cent.substring(2),16));
						else if (cent.startsWith("#"))
							sb.append(
								(char) Integer.parseInt(cent.substring(1)));
						// Insert custom entity definitions here
						else
							exc("Unknown entity: &" + cent + ";");
					} else
					{
						etag.append((char) c);
					}
					break;

					// we have just seen something like this:
					// <foo a="b"/
					// and are looking for the final >.
				case SINGLE_TAG :
					if (tagName == null)
						tagName = sb.toString();
					if (c != '>')
						exc("Expected > for tag: <" + tagName + "/>");
					doc.startElement(tagName, attrs);
					doc.text("");
					doc.endElement(tagName);
					if (depth == 0)
					{
						doc.endDocument();
						return;
					}
					sb.setLength(0);
					attrs = null;
					tagName = null;
					mode = popMode();
					break;

					// we are processing something
					// like this <foo ... >. It could
					// still be a <!-- ... --> or something.
				case OPEN_TAG :
					if (c == '>')
					{
						if (tagName == null)
							tagName = sb.toString();
						sb.setLength(0);
						depth++;
						doc.startElement(tagName, attrs);
						tagName = null;
						attrs = null;
						mode = popMode();
					} else if (c == '/')
					{
						mode = SINGLE_TAG;
					} else if (c == '-' && sb.toString().equals("!-"))
					{
						mode = COMMENT;
					} else if (c == '[' && sb.toString().equals("![CDATA"))
					{
						mode = CDATA;
						sb.setLength(0);
					} else if (c == 'E' && sb.toString().equals("!DOCTYP"))
					{
						sb.setLength(0);
						mode = DOCTYPE;
					} else if (Character.isWhitespace((char) c))
					{
						tagName = sb.toString();
						sb.setLength(0);
						mode = IN_TAG;
					} else
					{
						sb.append((char) c);
					}
					break;

					// We are processing the quoted right-hand side
					// of an element's attribute.
				case QUOTE :
					if (c == quotec)
					{
						rvalue = sb.toString();
						sb.setLength(0);
						attrs.addElement(new Object[]  {lvalue, rvalue});
						mode = IN_TAG;
						// See section the XML spec, section 3.3.3
						// on normalization processing.
					} else if (" \r\n\u0009".indexOf(c) >= 0)
					{
						sb.append(' ');
					} else if (c == '&')
					{
						pushMode(mode);
						mode = ENTITY;
						etag.setLength(0);
					} else
					{
						sb.append((char) c);
					}
					break;

				case ATTRIBUTE_RVALUE :
					if (c == '"' || c == '\'')
					{
						quotec = c;
						mode = QUOTE;
					} else if (Character.isWhitespace((char) c))
					{
						;
					} else
					{
						exc("Error in attribute processing");
					}
					break;

				case ATTRIBUTE_LVALUE :
					if (Character.isWhitespace((char) c))
					{
						lvalue = sb.toString();
						sb.setLength(0);
						mode = ATTRIBUTE_EQUAL;
					} else if (c == '=')
					{
						lvalue = sb.toString();
						sb.setLength(0);
						mode = ATTRIBUTE_RVALUE;
					} else
					{
						sb.append((char) c);
					}
					break;

				case ATTRIBUTE_EQUAL :
					if (c == '=')
					{
						mode = ATTRIBUTE_RVALUE;
					} else if (Character.isWhitespace((char) c))
					{
						;
					} else
					{
						exc("Error in attribute processing.");
					}
					break;

				case IN_TAG :
					if (c == '>')
					{
						mode = popMode();
						doc.startElement(tagName, attrs);
						depth++;
						tagName = null;
						attrs = null;
					} else if (c == '/')
					{
						mode = SINGLE_TAG;
					} else if (Character.isWhitespace((char) c))
					{
						;
					} else
					{
						mode = ATTRIBUTE_LVALUE;
						sb.append((char) c);
					}
					break;
			}
		}
		if (mode == DONE)
			doc.endDocument();
		else
			exc("missing end tag");
	}
	private void exc(String s) throws Exception
	{
		throw new Exception(s);
	}
}
