/*
 * Created on Jun 24, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import java.util.Enumeration;
import java.util.Hashtable;

import com.instant802.cldc.util.Log;

/**
 * @author alan
 *
 * Maintains the state of  EAP-based authentication sessions. 
 * Since EAP ids are only managed on a per supplciant basis we 
 * maintain our global notion of an EAP Session id here. 
 * 
 */
public class EAPSessionTable
{
	private final int GC_SLOT_LIMIT = 32;
	// Number of open session entries beyond which we try a table gc

	private Hashtable sessiontable;

	static private int key = 0;

	public EAPSessionTable()
	{
		sessiontable = new Hashtable();
	}

	public EAPState getEAPState(RadiusPacket rp)
	{
		byte[] key = rp.getState();
		EAPState state;

		if (null == key)
		{
			return null;
		}
		else
		{
			return (EAPState) sessiontable.get(new String(key));
		}
	}

	public EAPState createEAPState()
	{
		EAPState eapstate;
		String geid;

		gc();
		synchronized (this.getClass())
		{
			geid = "I802EapSession" + new Integer(key++).toString();
		}
		eapstate = new EAPState(geid.getBytes(), sessiontable);
		sessiontable.put(geid, eapstate);
		return eapstate;
	}

	/**
	 * A poor man's gc. Basically we want to sweep the table from time to time looking
	 * for stale entries. We only need to do this on eap session creation - and we don't want to do it
	 * every time. So we check to see if  the table is growing large and if it is sweep stale entries.
	 * This should be good enough and doesn't require a gc background thread.
	 */
	private void gc()
	{
		int gccount = 0, count = 0;
		if (sessiontable.size() > GC_SLOT_LIMIT)
		{
			if (Log.DEBUG) Log.debug("Starting EAP table gc sweep");
			Enumeration e = sessiontable.elements();
			EAPState state;

			while (e.hasMoreElements())
			{
				state = (EAPState) e.nextElement();
				++ count;
				if (state.isStale()) {
					state.drop();
					++gccount;
				}
			}
			if (Log.DEBUG) Log.debug("Removed " + gccount + " stale EAP entries out of " + count);
		}
	}
}
