/*
 * Created on Jun 27, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 * 
 * @author alanl
 */
package com.instant802.cldc.radius;

import com.instant802.cldc.ssl.*;
import com.instant802.cldc.util.Log;

import java.util.Hashtable;

/**
 * @author alan
 *
 * Code is MD5 CHallenge for now ...
 */

public class MSCHAPV2Method
{
	private RadiusServer server;
	private byte[] user;
	private byte[] secret;

	public final static int CHAP_CHALLENGE = 1, CHAP_RESPONSE = 2;

	// hash table of challenges sent to users. Since this radius server is embedded
	// we needn't bother ourselves with tracking addresses of NASs.
	private static Hashtable userchallenges = new Hashtable();

	public MSCHAPV2Method(RadiusServer server, byte[] user, byte[] secret)
	{
		if (Log.DEBUG) Log.debug("Construct MSCHAPV2Method: server = " + server +
		" user = " + user + " secret = " + secret);
		this.server = server;
		this.user = user;
		this.secret = secret;
	}

	/**
	 * 
	 * @param response - contains 49 byte MSchapV2 response
	 * 16 byte Peer Challenge, 8 bytes 0, 24 bytes NT response, 1 byte 0
	 * @param offset - offset into response where MSChapV2 response begins
	 * @return
	 */
	public boolean verifyChallenge(byte[] response, byte chapid, int off, MSChapV2VerifyChallengeResponse vcr)
	{
		if (Log.DEBUG) Log.info("Verifying challenge with response: " + response);
		if (response.length - off < 49)
		{
			if (Log.DEBUG) Log.info("Bad MSChapV2 response packet length: " + (response.length - off));
			return false;
		}
		byte[] authenticatorchallenge = (byte[]) userchallenges.get(new String(user));
		byte[] hashedpassword = (byte[]) server.getNTHashedPassword(user);
		byte[] genntresp;

		if (null == authenticatorchallenge)
		{
			if (Log.DEBUG) Log.info("MSChapV2: verifyChallenge - Null authenticatorchallenge ");
			return false;
		}
		if (null == hashedpassword)
		{
			if (Log.DEBUG) Log.info("MSChapV2: verifyChallenge - Null secret ");
			return false;
		}

		byte[] peerchallenge = new byte[16];
		System.arraycopy(response, off, peerchallenge, 0, 16);
		try
		{	
			genntresp = generateNTResponse(authenticatorchallenge, peerchallenge, user, hashedpassword);
			boolean success =  Utils.compareBytes(genntresp, 0, response, 24 + off, 24);
			vcr.setSuccess(success);
			vcr.setAuthenticatorChallenge(authenticatorchallenge);
			vcr.setNtResponse(genntresp);
			vcr.setHashedPassword(hashedpassword);
			vcr.setPeerChallenge(peerchallenge);
			vcr.setUserName(user);
			vcr.setChapid(chapid);
			return success;
		}
		catch (Exception e)
		{
			if (Log.DEBUG) Log.info("generateNTResponse: threw Exception: " + e.toString());
			return false;
		}
	}

	private byte[] generateChapChallenge(byte[] user)
	{
		byte[] challenge = new byte[16];
		long ts = System.currentTimeMillis();
		byte[] buf = new byte[8 + user.length];

		System.arraycopy(user, 0, buf, 8, user.length);

		for (int i = 0; i < 8; ++i)
		{
			buf[i] = (byte) (0xff & (ts >> (8 * i)));
		}
		new OpenSSL().md5(buf, buf.length, challenge);
		userchallenges.put(new String(user), challenge);
		return challenge;
	}

	/**
	 * Unlike MD5 (cf CHAPMethod) MSCHapV2 embeds the whole CHAP packet in its reponse.
	 * @param eapid
	 * @param eapmethod Sometime MS_EAP, sometimes MS_CHAP_V2 - go figure, as Mr Aboba puts it.
	 * @return
	 */
	public EAPPacket doChapChallenge(byte eapid, int eapmethod)
	{
		String sysname = "I802AP";
		CHAPPacket chapp =
			new CHAPPacket(CHAPPacket.CHAP_CHALLENGE, generateChapChallenge(this.user), (byte) 16,
			               sysname.getBytes(), sysname.length());
		if (Log.DEBUG) Log.debug(
			"Issuing MSCHAPV2 challenge to user  "
				+ ((null != user) ? new String(user, 0, user.length) : "<Unknown>")); 

		/** It *looks* as though there is a bug in MS XP/SP1 supplicant and
		 * it creates an EAP packet response to the MSCHapV2 challenge but sets
		 * its eapid to the *CHAP-ID* contained in this chap packet.
		 * 
		 * So, although we compute a nice unique chapid internally in the constructor
		 * we ram its id to be that of the EAP packet.
		 * 
		 * THis cost me 4 days of reverse engineering a MS bug -- please bill
		 * Redmond's Bill accordingly
		 */
		chapp.setId(eapid);
		// END OF HACK
		EAPPacket eapreply =
			new EAPPacket(
				EAPHandler.EAP_CODE_REQUEST,
				eapmethod,
				eapid,
				chapp.getData());
		return eapreply;
	}

	// --------------------- Support Methods from RFC 2759

	private byte[] generateNTResponse(
		byte[] authenticatorChallenge,
		byte[] peerChallenge,
		byte[] userName,
		byte[] passwordHash)
		throws SSLParameterException
	{
		byte[] challenge = new byte[8];
		challenge = challengeHash(peerChallenge, authenticatorChallenge, userName);
		return challengeResponse(challenge, passwordHash);
	}

	private byte[] challengeHash(
		byte[] peerChallenge,
		byte[] authenticatorChallenge,
		byte[] userName)
		throws SSLParameterException
	{
		byte[] challenge = new byte[8];
		byte[] digest = new byte[20];
		SHA1 sha1;

		sha1 = new SHA1();

		sha1.update(peerChallenge, 16);
		sha1.update(authenticatorChallenge, 16);
		// Drop any domain name part -- see 8.2 in RFC 2759
		String uname = new String(userName);
		uname = uname.substring(uname.indexOf('\\') + 1);
		userName = uname.getBytes();
		sha1.update(userName, userName.length);
		sha1.finale(digest);
		System.arraycopy(digest, 0, challenge, 0, 8);
		return challenge;
	}

//	private byte[] ntPasswordHash(byte[] password)
//	{
//		byte[] digest = new byte[16];
//		new OpenSSL().md4(password, password.length, digest);
//		return digest;
//	}

	private byte[] hashNtPasswordHash(byte[] passwordHash)
	{
		byte[] digest = new byte[16];

		new OpenSSL().md4(passwordHash, passwordHash.length, digest);
		return digest;
	}

	private byte[] challengeResponse(byte[] challenge, byte[] passwordHash)
	{
		byte[] response = new byte[24];
		byte[] zpasswordHash = new byte[21];
		byte[] desout;

		System.arraycopy(passwordHash, 0, zpasswordHash, 0, 16);

		for (int i = 16; i < 21; ++i)
		{
			zpasswordHash[i] = 0;
		}

		for (int i = 0; i < 3; ++i)
		{
			desout = desEncrypt(challenge, zpasswordHash, i * 7);
			System.arraycopy(desout, 0, response, i * 8, 8);
		}
		return response;
	}

	private byte[] desEncrypt(byte[] clear, byte[] key7, int keyoff)
	{
		byte[] cypher = new byte[8];
		byte[] key8 = addParity(key7, keyoff);
		DES des = new DES(key8);
		des.encrypt(clear, cypher);
		return cypher;
	}

	public byte[] generateAuthenticatorResponse(
		byte[] passwordHash,
		byte[] ntResponse,
		byte[] peerChallenge,
		byte[] authenticatorChallenge,
		byte[] userName)
		throws SSLParameterException
	{
		byte[] passwordHashHash = new byte[16];
		byte[] challenge = new byte[8];
		byte[] digest = new byte[20];
		SHA1 sha1;

		sha1 = new SHA1();

		byte magic1[] =
			new byte[] {
				0x4D,
				0x61,
				0x67,
				0x69,
				0x63,
				0x20,
				0x73,
				0x65,
				0x72,
				0x76,
				0x65,
				0x72,
				0x20,
				0x74,
				0x6F,
				0x20,
				0x63,
				0x6C,
				0x69,
				0x65,
				0x6E,
				0x74,
				0x20,
				0x73,
				0x69,
				0x67,
				0x6E,
				0x69,
				0x6E,
				0x67,
				0x20,
				0x63,
				0x6F,
				0x6E,
				0x73,
				0x74,
				0x61,
				0x6E,
				0x74 };

		byte magic2[] =
			new byte[] {
				0x50,
				0x61,
				0x64,
				0x20,
				0x74,
				0x6F,
				0x20,
				0x6D,
				0x61,
				0x6B,
				0x65,
				0x20,
				0x69,
				0x74,
				0x20,
				0x64,
				0x6F,
				0x20,
				0x6D,
				0x6F,
				0x72,
				0x65,
				0x20,
				0x74,
				0x68,
				0x61,
				0x6E,
				0x20,
				0x6F,
				0x6E,
				0x65,
				0x20,
				0x69,
				0x74,
				0x65,
				0x72,
				0x61,
				0x74,
				0x69,
				0x6F,
				0x6E };

		passwordHashHash = hashNtPasswordHash(passwordHash);

		sha1.update(passwordHashHash, 16);
		sha1.update(ntResponse, 24);
		sha1.update(magic1, 39);
		sha1.finale(digest);

		challenge = challengeHash(peerChallenge, authenticatorChallenge, userName);

		sha1 = new SHA1();
		sha1.update(digest, 20);
		sha1.update(challenge, 8);
		sha1.update(magic2, 41);
		sha1.finale(digest);

		//
		// Encode the value of 'Digest' as "S=" followed by
		// 40 ASCII hexadecimal digits and return it in
		// AuthenticatorResponse.
		// For example,
		//   "S=0123456789ABCDEF0123456789ABCDEF01234567"
		//
		StringBuffer resp = new StringBuffer("S=");
		resp.append(Utils.hexstr(digest, 20));
		if (Log.DEBUG) Log.debug("Authenticator response = " + resp.toString());
		return resp.toString().getBytes();

	}
	/*	
			private boolean checkAuthenticatorResponse(
				byte[] password,
				byte[] ntResponse,
				byte[] peerChallenge,
				byte[] authenticatorChallenge,
				byte[] userName,
				byte[] receivedResponse)
				throws SSLParameterException
			{
		
				byte[] myResponse;
		
				boolean responseOK = false;
				myResponse =
					generateAuthenticatorResponse(
						password,
						ntResponse,
						peerChallenge,
						authenticatorChallenge,
						userName);
				if (myResponse.length == receivedResponse.length)
				{
					responseOK = true;
					for (int i = 0; i < myResponse.length; ++i)
						if (myResponse[i] != receivedResponse[i])
						{
							responseOK = false;
							break;
						}
				}
				return responseOK;
			}
		
			private PWBlock newPasswordEnryptedWirhOldNtPasswordHash(
				byte[] newPassword,
				byte[] oldPassword)
			{
				byte[] passwordHash;
				PWBlock encryptedPWBlock;
		
				passwordHash = ntPasswordHash(oldPassword);
				encryptedPWBlock = encryptPwBlockWithPasswordHash(newPassword, passwordHash);
		
				return encryptedPWBlock;
		
			}
		
			private PWBlock encryptPwBlockWithPasswordHash(byte[] password, byte[] passwordHash)
			{
				PWBlock clearPwBlock;
		
				clearPwBlock = new PWBlock();
				int pwsize;
		
				for (int i = 0; i < 256; ++i)
				{
					clearPwBlock.password[i] = (byte) (System.currentTimeMillis() & 0xFF); // cheap  
				}
		
				System.arraycopy(
					password,
					0,
					clearPwBlock.password,
					256 - password.length,
					password.length);
				clearPwBlock.setPasswordLength(password.length);
		
				return rc4Encrypt(clearPwBlock.password, 260, passwordHash, passwordHash.length);
			}
		
			private PWBlock rc4Encrypt(byte[] clear, int clearlen, byte[] key, int keylen)
			{
		
				RC4 rc4 = new RC4(key, keylen);
				byte[] cypher = new byte[clearlen];
				PWBlock pwBlock;
		
				rc4.encrypt(clear, clearlen, cypher);
		
				pwBlock = new PWBlock(cypher, clearlen);
				return pwBlock;
			}
		
			private byte[] oldNtPasswordHashEncryptedWithNewNtPasswordHash(
				byte[] newPassword,
				byte[] oldPassword)
			{
				byte[] oldPasswordHash, newPasswordHash;
		
				oldPasswordHash = ntPasswordHash(oldPassword);
				newPasswordHash = ntPasswordHash(newPassword);
		
				return ntPasswordHashEncryptedWithBlock(oldPasswordHash, newPasswordHash);
			}
		
			private byte[] ntPasswordHashEncryptedWithBlock(byte[] passwordHash, byte[] block)
			{
				byte[] cypher = new byte[16];
				byte[] out = new byte[8];
		
				DES des;
		
				des = new DES(block, 0, 7);
		
				des.encrypt(passwordHash, 0, 8, out);
				System.arraycopy(out, 0, cypher, 0, 8);
		
				des = new DES(block, 7, 7);
		
				des.encrypt(passwordHash, 8, 8, out);
				System.arraycopy(out, 0, cypher, 8, 8);
		
				return cypher;
			}
		
		*/

	/** Takes a 7-byte quantity and returns a valid 8-byte DES key.
	 * The input and output bytes are big-endian, where the most significant
	 * byte is in element 0. 
	 * @param off -- offset in in key begins at
	 * @param in  -- buffer contating 7 octet segment starting at off
	 */
	public static byte[] addParity(byte[] in, int off)
	{
		byte[] result = new byte[8];

		// Keeps track of the bit position in the result
		int resultIx = 1;

		// Used to keep track of the number of 1 bits in each 7-bit chunk
		int bitCount = 0;

		// Process each of the 56 bits
		for (int i = 0; i < 56; i++)
		{
			// Get the bit at bit position i
			boolean bit = (in[off + (6 - i / 8)] & (1 << (i % 8))) > 0;

			// If set, set the corresponding bit in the result
			if (bit)
			{
				result[7 - resultIx / 8] |= (1 << (resultIx % 8)) & 0xFF;
				bitCount++;
			}

			// Set the parity bit after every 7 bits
			if ((i + 1) % 7 == 0)
			{
				if (bitCount % 2 == 0)
				{
					// Set low-order bit (parity bit) if bit count is even
					result[7 - resultIx / 8] |= 1;
				}
				resultIx++;
				bitCount = 0;
			}
			resultIx++;
		}
		return result;
	}


	// ----------------------- End of Support methods
}

/*
class PWBlock
{

	public byte[] password;

	public PWBlock()
	{
		password = new byte[260]; // 256 byte pw + 4 bytes length a la c-struct
	}

	public PWBlock(byte[] buff, int bufflen)
	{
		password = new byte[bufflen + 4];
		System.arraycopy(buff, 0, password, 0, bufflen);
		setPasswordLength(bufflen);
	}

	public void setPasswordLength(int len)
	{
		for (int i = 1; i <= 4; ++i)
		{
			password[260 - i] = (byte) ((len >> (8 * (i - 1)) & 0xff));
		}
	}
}
*/
