/*******************************************************************************
 * 
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import java.util.Hashtable;
import java.util.Vector;
import com.instant802.cldc.ssl.SSLMem;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class PEAPMethod {
	RadiusServer server;
	byte[] secret, user;
	private static Hashtable fragtable = new Hashtable();
	private final int TLS_BUFFER_SIZE = 2048;

	public PEAPMethod(RadiusServer server, byte[] user, byte[] secret) {
		this.server = server;
		this.secret = secret;
		this.user = user;
	}

	private void debug(String msg) {
		if (Log.DEBUG) Log.debug("PEAPMethod: " + msg);
	}

	public int doPeapResponse(
		EAPPacket eap,
		EAPState eapstate,
		RadiusResponse resp,
		byte[] requestauthenticator) {

		PEAPPacket pp = new PEAPPacket(eap.getTypeData());
		byte id = eap.getId();
		Vector info = null;
		byte[] tlsdata = null;
		int eapcode = EAPHandler.EAP_CODE_INVALID;
		boolean lbit, mbit;

		lbit = 0 != (pp.getFlags() & PEAPPacket.LBIT);
		mbit = 0 != (pp.getFlags() & PEAPPacket.MBIT);

		if (!(mbit || lbit)) {
			// Self-contained
			tlsdata = pp.getTLSData();
		}
		else {
			info = (Vector) fragtable.get(eapstate.getId());
			if (null == info) {
				// We don't seem to know about this PEAP session
				if (!lbit) {
					// We came in midway -- error out
					eapstate.setEventdetail(
						EAPState.ESED_PEAP_PROTOCOL_FAILURE);
					return EAPHandler.EAP_CODE_FAILURE;
				}
				else {
					if (Log.DEBUG) debug("Start of fragmented PEAP message");
					// Start of a fragmented message
					info = new Vector();
					fragtable.put(eapstate.getId(), info);
				}
			}
			// Add the packet to the fragment list
			int msglength = pp.getMsgLength();
			info.addElement(new Integer(msglength));
			info.addElement(pp.getTLSData());
			// Are we done
			if (!mbit) {
				if (Log.DEBUG) debug("Last fragment of fragmented message");
				tlsdata = new byte[((Integer) info.elementAt(0)).intValue()];
				int curlen = 0;
				for (int i = 1; i < info.size(); ++i) {
					byte[] segment = (byte[]) info.elementAt(i);
					System.arraycopy(
						segment,
						0,
						tlsdata,
						curlen,
						segment.length);
					curlen = curlen + segment.length;
				}
				fragtable.remove(eapstate.getId());
			}
			else {
				if (Log.DEBUG) debug("Expecting more PEAP fragments in message");
				// Ack the packet and we are done
				new EAPPacket(
					EAPHandler.EAP_CODE_REQUEST,
					EAPHandler.EAP_TYPE_PEAP,
					(byte) (id + 1),
					new byte[0]).embedInRadiusResponse(
					resp);		
				return EAPHandler.EAP_CODE_REQUEST;
			}
		}
		if (null != tlsdata) {
			eapcode =
				processTLSPacket(
					tlsdata,
					pp.getVersion(),
					id,
					eapstate,
					resp,
					requestauthenticator);
		}
		else {
			if (Log.DEBUG) Log.debug("PeapMethod: doPeapResponse found no TLSData");
			eapstate.setEventdetail(EAPState.ESED_PEAP_PROTOCOL_FAILURE);
			eapcode = EAPHandler.EAP_CODE_FAILURE;
		}
		return eapcode;
	}

	/**
	 * We have assembled the TLS Data packet - now process it and generate the
	 * EAP reply packet
	 * 
	 * @param tlsdata
	 * @param id
	 * @return eapcode
	 */
	private int processTLSPacket(
		byte[] tlsdata,
		byte peapversion,
		byte eapid,
		EAPState eapstate,
		RadiusResponse resp,
		byte[] requestauthenticator) {
		SSLMem sslfilter = null;
		int eapcode = EAPHandler.EAP_CODE_INVALID;
		byte tlsbuffer[] = new byte[TLS_BUFFER_SIZE];

		if (Log.DEBUG) Log.debug(
			"In processTLSPacket - EAP Session Id = "
				+ new String(eapstate.getId())
				+ " EAPState = "
				+ eapstate.getState());

		if (0 == tlsdata.length) // Should only occur at end of
			// phase 1
			{
			if (EAPState.EAP_STATE_PEAP_REQUEST_PHASE1
				== eapstate.getState()) {
				byte[] recv = new byte[32];
				byte[] send = new byte[32];
				try {
					// and get the keys
					sslfilter = eapstate.getSslfilter();
					sslfilter.getRecvSendKeys(recv, send);
					eapstate.setMpperecvkey(recv);
					eapstate.setMppesendkey(send);
				}
				catch (Exception e) {
					if (Log.DEBUG) Log.info("Cannot initialize PEAP tunnel: " + e.toString());
					eapstate.setEventdetail(
						EAPState.ESED_PEAP_PROTOCOL_FAILURE);
					return EAPHandler.EAP_CODE_FAILURE;
				}

				if (Log.DEBUG) Log.debug(
					"Sending EAP Identity through negotiated TLS channel (start PEAP phase II)");
				eapstate.setState(EAPState.EAP_STATE_PEAP_REQUEST_PHASE2);
				// Send back Identity request through TLS channel
				EAPPacket eapresp =
					new EAPPacket(
						EAPHandler.EAP_CODE_REQUEST,
						EAPHandler.EAP_TYPE_IDENTITY,
						(byte) (eapid + 1));		
				sslfilter.write(getTunnelData(eapresp, peapversion));
				eapcode = EAPHandler.EAP_CODE_REQUEST;
				// To be picked up by pending below
			}
			else {
				if (Log.DEBUG) Log.info(
					"PEAP: Null TLS data not in PEAP_PHASE1 ("
						+ eapstate.getState()
						+ ")");
			}
		}
		else {
			if (null == (sslfilter = eapstate.getSslfilter())) {
				// Set up an sslfilter for the tunnel
				if (Log.DEBUG) debug("Setting up SSLMem filter");
				sslfilter = new SSLMem(server.getSSLSessionContext());
				eapstate.setSslfilter(sslfilter);
			}
			int bytesread = sslfilter.read(tlsdata, tlsbuffer);
			if  (bytesread < 0) {
				/*
				 * If we are still negotiating the tunnel then we can expect to
				 * get WANT_READ errors and we should should just press on -
				 * otherwise an error here is probably an error (?) and we
				 * should abort
				 */
				try{
					if (!(SSLMem.SSL_ERROR_WANT_READ
						== sslfilter.getError(bytesread)
						&& EAPState.EAP_STATE_PEAP_REQUEST_PHASE1
							== eapstate.getState())) {
						if (Log.DEBUG) Log.info(
							"Error reading tunneled EAP data: code= "
								+ sslfilter.getError(bytesread)
								+ "\n    EAP State is "
								+ eapstate.getState());
						server.getSSLSessionContext().printErrors();
						eapstate.setEventdetail(
							EAPState.ESED_PEAP_PROTOCOL_FAILURE);
						return EAPHandler.EAP_CODE_FAILURE;
					}
				} catch ( SocketException e  ) {
					if (Log.DEBUG) Log.info( "PEAPMethod", "returning EAP_CODE_FAILURE on SocketException" );
					return EAPHandler.EAP_CODE_FAILURE;
				}
			}
			else if  (bytesread > 0) {
				EAPPacket eapresp =
					processTunneledEAP(
						tlsbuffer,
						bytesread,
						eapstate,
						eapid,
						sslfilter,
						resp,
						requestauthenticator);
				if (null != eapresp) {
					if (Log.DEBUG) Log.debug("TLS tunnel has been pulled down");
					eapresp.embedInRadiusResponse(resp);
					eapcode = eapresp.getCode();
					return eapcode;
				}
			}
			else {
				if (Log.DEBUG) Log.debug(
					">> Failed to read any tlsdata: " + Utils.hex(tlsdata));
			}
		}
		// Has reading or writing caused a need for bytes to be send back ??
		if (Log.DEBUG) debug("Checking for pending");
		int pending = sslfilter.pending();

		if (pending > 0) {
			if (Log.DEBUG) debug("Dealing with pending data");
			int sendbytes = 0;
			while (pending > 0) {
				sendbytes = sslfilter.readPeerChannel(tlsbuffer);
				pending = sslfilter.pending();
				if (pending > 0) {
					if (Log.DEBUG) Log.info("PEAPMethod: return handshake too large !!!");
					// temp limitation
					eapstate.setEventdetail(
						EAPState.ESED_PEAP_PROTOCOL_FAILURE);
					return EAPHandler.EAP_CODE_FAILURE;
				}
			}
			byte[] newbuf = new byte[sendbytes + 1];
			newbuf[0] = PEAPPacket.VERSION;
			System.arraycopy(tlsbuffer, 0, newbuf, 1, sendbytes);
			// hack for now
			new EAPPacket(
				EAPHandler.EAP_CODE_REQUEST,
				EAPHandler.EAP_TYPE_PEAP,
				(byte) (eapid + 1),
				newbuf).embedInRadiusResponse(
				resp);
			eapcode = EAPHandler.EAP_CODE_REQUEST;
			if (Log.DEBUG) Log.debug(
				"Replying with EAP_CODE_REQUEST: EAP_TYPE_PEAP: EAP-ID="
					+ (eapid + 1));
		}
		else {
			// This should never happen ??
			if (Log.DEBUG) Log.info("No pending data for TLS reply");
			eapstate.setEventdetail(EAPState.ESED_PEAP_PROTOCOL_FAILURE);
			eapcode = EAPHandler.EAP_CODE_FAILURE;
		}
		return eapcode;
	}

	/**
	 * Handle PEAP version 0 nonesense for data to be written
	 * 
	 * @param eapresp
	 * @param eapid
	 * @return
	 */
	private byte[] getTunnelData(EAPPacket eapresp, byte peapversion) {
		if (0x00 == peapversion
			&& EAPHandler.EAP_TYPE_EXTENSIONS != eapresp.getType())
			return eapresp.getData(); // Type and data
		else
			return eapresp.data; // whole packet
	}

	/**
	 * Create an EAP packet from the buffer pulled out of the tunnel. Again
	 * have to deal with PEAP ver 0 nonesense.
	 * 
	 * @param buff
	 * @param bufflength
	 * @param peapversion
	 * @param code
	 * @param id
	 * @return
	 */
//	private EAPPacket getTunneledPacket(
//		byte[] buff,
//		int bufflength,
//		byte peapversion,
//		byte code,
//		byte id) {
//		EAPPacket packet;
//
//		if (0xff == peapversion && EAPHandler.EAP_TYPE_EXTENSIONS != buff[0]) {
//			packet =
//				new EAPPacket(code, buff[0], (byte) (id + 1), buff, bufflength);
//		}
//		else
//			packet = new EAPPacket(buff, bufflength);
//		return packet;
//	}

	/**
	 * We have a _big_ issue here. MS SP1 and Funk support version '0' of PEAP
	 * which is an MS abortion. EAP packets are tunneled inside TLS _without_
	 * their headers _unless_ (are you listening ?) the EAP type is 33 (EAP
	 * Extension) in which case the entire header _is_ sent. In the former case
	 * the EAP identifier and Code fields are _assumed_ to be the same as the
	 * EAP packet bearing the TLS (which of course doesn't work). Length can be
	 * reconstructed easily enough. Usually this directly writes data back
	 * through the tunnel. However towards the end of the exchange (e.g. after
	 * the authentication phase) the tunnel is ripped down and an EAP-SUCCESS
	 * or EAP-FAILURE should be returned directly in the radius Packet as a
	 * response - this packet is in the return value.
	 * 
	 * TODO: This method is in bad need of refactoring since the logic for
	 * different tunneled protocols (e.g. MSChapV2 is embedded here). The best
	 * way (?) would be to determine the tunneled protocol, grab a handler and
	 * then store the handler away in the EAPState structure.
	 * 
	 * @param inbuf
	 * @param bytesread
	 * @param eapstate
	 * @param sslfilter
	 */
	private EAPPacket processTunneledEAP(
		byte[] inbuf,
		int bytesread,
		EAPState eapstate,
		byte eapid,
		SSLMem sslfilter,
		RadiusResponse radiusresp,
		byte[] requestauthenticator) {
		EAPPacket resp = null;
		byte type;

		if (Log.DEBUG) Log.debug("PEAP Inner read: " + Utils.hexstr(inbuf, bytesread));
		type = inbuf[0];
		switch (eapstate.getState()) {
			case EAPState.EAP_STATE_PEAP_REQUEST_PHASE2 :

				// Propose MSChapV2
				if (EAPHandler.EAP_TYPE_IDENTITY == type) {
					if (Log.DEBUG) Log.info("Proposing MSChapV2/PEAP EAP_TYPE_MS_EAP");
					resp =
						new MSCHAPV2Method(
							this.server,
							this.user,
							this.secret).doChapChallenge(
							(byte) (eapid + 1),
							EAPHandler.EAP_TYPE_MS_EAP);
					eapstate.setState(EAPState.EAP_STATE_PEAP_REQUEST_MSCHAPV2);				
				}
				else {
					if (Log.DEBUG) Log.info(
						"*** Got EAP type: "
							+ inbuf[0]
							+ " in EAP_STATE_PEAP_REQUEST_PHASE2");
				}
				break;

			case EAPState.EAP_STATE_PEAP_REQUEST_MSCHAPV2 :

				if (EAPHandler.EAP_TYPE_NAK == type)
					// Negotiate down to EAP/MD5 for testing
					{
					if (inbuf.length >= 2)
						if (inbuf[1] == EAPHandler.EAP_TYPE_MD5) {
							if (Log.DEBUG) Log.info("Negotiating MD5");
							resp =
								new CHAPMethod(
									this.server,
									this.user,
									this.secret).doChapChallenge(
									eapid);
							eapstate.setState(
								EAPState.EAP_STATE_PEAP_REQUEST_MD5);
							break;
						}
						else if (inbuf[1] == EAPHandler.EAP_TYPE_MSCHAPV2) {
							if (Log.DEBUG) Log.info(
								"Proposing MSChapV2/PEAP on EAP_TYPE_MSCHAPV2");
							resp =
								new MSCHAPV2Method(
									this.server,
									this.user,
									this.secret).doChapChallenge(
									eapid,
									EAPHandler.EAP_TYPE_MSCHAPV2);
							eapstate.setState(
								EAPState.EAP_STATE_PEAP_REQUEST_MSCHAPV2);
						}
						else {
							if (Log.DEBUG) Log.info(
								"*** Got NAK/EAP type: "
									+ inbuf[0]
									+ " in EAP_STATE_PEAP_REQUEST_MSCHAPV2");
						}
				}
				else if (
					EAPHandler.EAP_TYPE_MSCHAPV2 == type
						|| EAPHandler.EAP_TYPE_MS_EAP == type) {
					/**
					 * MSCHapV2 is embedded as value in Chap Packet
					 */
					if (Log.DEBUG) Log.info("Verifying challenge response");
					MSChapV2VerifyChallengeResponse vcr =
						new MSChapV2VerifyChallengeResponse();
					if (Log.DEBUG) Log.info("Got response: " + vcr);
					boolean success =
						new MSCHAPV2Method(
							this.server,
							this.user,
							this.secret).verifyChallenge(
							inbuf,
							inbuf[2],
							6,
							vcr);
					if (Log.DEBUG) debug(
						"Got MSCHapV2 result: "
							+ success
							+ " eapstate = "
							+ eapstate);
					try {
						eapstate.setEventdetail(EAPState.ESED_MSCHAPV2);
						if (success) {
							if (Log.DEBUG) debug("User passed MSChapV2 challenge");
							eapstate.setState(
								EAPState.EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_1);
						}
						else {
							if (Log.DEBUG) debug("User failed MSChapV2 challenge");
							eapstate.setState(
								EAPState.EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_1);
						}
						CHAPPacket chapresp = vcr.generateResponsePacket();
						resp =
							new EAPPacket(
								EAPHandler.EAP_CODE_REQUEST,
								EAPHandler.EAP_TYPE_MS_EAP,
								(byte) (eapid + 1),
								chapresp.getData());
					}
					catch (Exception e) {
						if (Log.DEBUG) Log.info(
							"Exception during EAP_STATE_PEAP_REQUEST_MSCHAPV2:"
								+ e.toString());
						eapstate.setEventdetail(EAPState.ESED_INTERNAL_ERROR);
						return new EAPPacket(
							EAPHandler.EAP_CODE_FAILURE,
							(byte) (eapid + 1));
					}

					break;
				}
				else {
					if (Log.DEBUG) Log.info("Strange EAP response code to MsChapV2 offer");
					eapstate.setEventdetail(
						EAPState.ESED_PEAP_PROTOCOL_FAILURE);
				}

				break;

			case EAPState.EAP_STATE_PEAP_REQUEST_MD5 :
				if (EAPHandler.EAP_TYPE_MD5 == type) {
					if (inbuf.length >= 2)
						if (inbuf[1] == 16) // Length of response
							// - MD5 hash
							{
							byte[] typedata = new byte[inbuf.length - 2];
							System.arraycopy(
								inbuf,
								2,
								typedata,
								0,
								inbuf.length - 2);
							resp =
								new CHAPMethod(
									server,
									user,
									secret).doChapResponse(
									typedata,
									eapid);
							break;
						}
				}
				else {
					if (Log.DEBUG) Log.info("Strange EAP response code to MD5 offer");
					eapstate.setEventdetail(
						EAPState.ESED_PEAP_PROTOCOL_FAILURE);
				}
				break;

			case EAPState.EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_1 :
			case EAPState.EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_1 :
				// We have succ/failed at the MSCHap V2 level -- now we do it
				// at the
				// PEAP v0 level using AVP inside a EAP Extns packet
				boolean success =
					EAPState.EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_1
						== eapstate.getState();
				AVP avp = AVP.makeResult(success);
				resp =
					new EAPPacket(
						EAPHandler.EAP_CODE_REQUEST,
						EAPHandler.EAP_TYPE_EXTENSIONS,
						(byte) (eapid + 1),
						avp.getData());
				eapstate.setState(
					success
						? EAPState.EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_2
						: EAPState.EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_2);
				break;

			case EAPState.EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_2 :
			case EAPState.EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_2 :
				/**
				 * We should be getting EAP Extension packets back -- so we
				 * have the full EAP header (thanks MS) hence our type is in
				 * fact an EAP RESPONSE.
				 */
				if (EAPHandler.EAP_CODE_RESPONSE == type) {
					AVP result = new AVP(inbuf, 5, 5);
					if (result.isResult()) {
						if (result.isSuccess()) {
							radiusresp.addMppeKeys(
								eapstate,
								secret,
								requestauthenticator);
						}
						if (Log.DEBUG) debug(
							"Sending final ** "
								+ (result.isSuccess()
									? " success "
									: " failure "));					
						return new EAPPacket(
							result.isSuccess()
								? EAPHandler.EAP_CODE_SUCCESS
								: EAPHandler.EAP_CODE_FAILURE,
							(byte) (eapid + 1));
					}
					else {
						if (Log.DEBUG) Log.info("Got non-result EAPExtension packet");
					}
				}
				else {
					if (Log.DEBUG) Log.info(
						"Got strange type in EAP_STATE_PEAP_AUTHENTICATED:"
							+ type);
				}
				eapstate.setEventdetail(EAPState.ESED_PEAP_PROTOCOL_FAILURE);
				return new EAPPacket(
					EAPHandler.EAP_CODE_FAILURE,
					(byte) (eapid + 1));

			default :
				if (Log.DEBUG) Log.info(
					"Unknown EAPState in processTunnelEAP: "
						+ eapstate.getState());
				eapstate.setEventdetail(EAPState.ESED_PEAP_PROTOCOL_FAILURE);
				return EAPPacket.failurePacket((byte) (eapid + 1));
		}
		if (null != resp) {
			if (resp.getType() == EAPHandler.EAP_TYPE_EXTENSIONS)
				sslfilter.write(resp.data);
			else
				sslfilter.write(resp.getData()); // Strip
			// off
			// header
			// ??
		}
		return null;
	}
}
