package com.instant802.cldc.radius;

import com.instant802.cldc.radius.session.AuthenticationHistory;
import com.instant802.cldc.radius.session.AuthenticationHistoryImpl;
import com.instant802.cldc.deviceconfiguration.ConfigurationModel;
import com.instant802.cldc.util.ControlledQueue;
import com.instant802.cldc.util.Log;

import com.instant802.cldc.net.UDPDatagramSocket;
import com.instant802.cldc.net.DatagramSocket;

/**
 * @author robin, alanl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class Proxy implements Runnable
{
	public static boolean running;
	public static DatagramSocket socket;
	public static ControlledQueue receiveQueue;
	public static ControlledQueue sendQueue;
	public static boolean debug;
	public RadiusTransactionManager forwarder;

	int port;
	byte[] buf;
	RadiusServer server;
	static ConfigurationModel apconfig;
	AuthenticationHistory authhist;

	private Thread receiver, sender, processor;
	
	public Proxy(int port, boolean debug, ConfigurationModel apconfig)
	{
		if (Log.DEBUG) Log.debug("Starting proxy server");
		this.port = port;
		authhist = new AuthenticationHistoryImpl();		
		forwarder = new RadiusTransactionManager();
		forwarder.start();
		Proxy.receiveQueue = new ControlledQueue();
		Proxy.sendQueue = new ControlledQueue();
		server = new RadiusServer(this, apconfig);
		Proxy.running = true;
		Proxy.apconfig = apconfig; // Need to get log 
		// Proxy.debug = debug;
		Proxy.debug = true;
	}

	public RadiusServer getServer()
	{
		return server;
	}

	public RadiusTransactionManager getForwarder() {
		return forwarder;
	}

	public void run()
	{
		server.setupInitialUsers();
		try
		{
			socket = new UDPDatagramSocket(port);
		}
		catch (Exception ie)
		{
			Log.emergency("Radius: Cannot open socket @  " + port + ":" + ie.getMessage());
			System.exit(-1);
		}
		this.receiver = new Receiver();
		receiver.start();
		this.sender = new Sender();
		sender.start();
		this.processor = new Processor(this);
		processor.start();
	}
	
	public void waitForStop() {
		running = false;
		receiveQueue.close();
		sendQueue.close();
		try {
			if (Log.DEBUG) info("waiting for sender to stop");
			sender.join();
		} catch (InterruptedException e) {
			Log.error( e );
		}
		try {
			if (Log.DEBUG) info("waiting for receiver to stop");
			receiver.join();
		} catch (InterruptedException e1) {
			Log.error( e1 );
		}
		try {
			if (Log.DEBUG) info("waiting for processor to stop");
			processor.join();
		} catch (InterruptedException e2) {
			Log.error( e2 );
		}
		info("proxy returning from waitForStop");
	}
	
	private void info(String string) {
		Log.info( "Proxy", string );
	}

	public static void main(String[] args)
	{
		int port = 1812; // radius default

		for (int i = 0; i < args.length; ++i)
		{
			if ("-d".equals(args[i]))
				Proxy.debug = true;
			else
			{
				if ("-p".equals(args[i]))
				{
					++i;
					if (i < args.length)
						port = Integer.parseInt(args[0].trim());
				}
			}
		}
		new Proxy(port, Proxy.debug, null).run();
	}

	public AuthenticationHistory getAuthenticationHistory()
	{
		return authhist;
	}
}
