/*
 * Created on Apr 18, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.ssl.OpenSSL;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class RadiusResponse
{

	int code, id;
	Hashtable attribs;
	Object associatedpacket;

	public RadiusResponse(int code, int id)
	{
		this.code = code;
		this.id = id;
		attribs = new Hashtable();
		this.associatedpacket = null;
	}

	public RadiusResponse(int id)
	{
		this(RadiusPacket.R_INVALID, id);
	}

	public void setCode(int code)
	{
		this.code = code;
	}

	public void addAttribute(int attrcode, byte[] value)
	{
		Vector vals;
		Integer key = new Integer(attrcode);

		if (null == (vals = (Vector) attribs.get(key)))
		{
			vals = new Vector();
			attribs.put(key, vals);
		}
		vals.addElement(value);
	}

	/**
	 * Serialize the Radius Response packet (defined by attribs, code and id) to a radius packet.
	 * If autrheticate is true then we will ad a MSG-AUTHETHENTICATE attribute by computing the hmacmd5 
	 * signature according to rfc2869, We do them both here because in both cases you are basically creating
	 * the new packet and I am trying to be a (little) bit efficient at the cost of some clarity .. probably could be refactored 
	 * @param request
	 * @param secret
	 * @param authenticate
	 * @return
	 */
	public byte[] createResponsePacket(RadiusPacket request, byte[] secret, boolean authenticate)
	{
		byte authbuf[] = new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		Integer key = null;
		if (authenticate)
		{
			// Make sure we have the MSG_AUTHENT attrib and it is 0 to start with
			Vector vals =
				(Vector) attribs.get(key = new Integer(EAPHandler.ATTR_MSG_AUTHENTICATOR));
			if (null == vals)
			{
				vals = new Vector();
				attribs.put(key, vals);
			}
			vals.removeAllElements();
			vals.addElement(authbuf);
		}
		byte[] attribuf = RadiusPacket.makeAttrBuffer(attribs);
		byte[] response = new byte[20 + attribuf.length];
		byte[] md5buf = new byte[response.length + secret.length];
		byte[] md5hash = new byte[16];
		byte[] hmachash = new byte[16];

		response[0] = (byte) (0xff & code);
		response[1] = (byte) (0xff & id);
		response[2] = (byte) (0xff & (0xff00 & response.length) >> 8);
		response[3] = (byte) (0xff & response.length);
		// Copy the Request Autheticator into the authenticator field because we need to compute
		// the response authenticator which is MD5(code+id+length+reqauthent+atribs+secret)
		System.arraycopy(request.data, 4, response, 4, 16);
		// Now copy attributes over ..
		System.arraycopy(attribuf, 0, response, 20, attribuf.length);

		if (authenticate)
		{
			// compute HMAC
			new OpenSSL().hmacmd5(secret, secret.length, response, response.length, hmachash);
			// Put the sig back
			 ((Vector) attribs.get(key)).setElementAt(hmachash, 0);
			// Now we have to get the attributes back out since they have changed and we need to compute
			// the last hash _over_ the msg authenticator
			attribuf = RadiusPacket.makeAttrBuffer(attribs);
			System.arraycopy(attribuf, 0, response, 20, attribuf.length);
		}

		System.arraycopy(response, 0, md5buf, 0, response.length);
		System.arraycopy(secret, 0, md5buf, response.length, secret.length);
		new OpenSSL().md5(md5buf, response.length + secret.length, md5hash);
		System.arraycopy(md5hash, 0, response, 4, 16);
		return response;
	}

	private void addMppeKey(byte[] key, byte[] secret, byte[] requestauthenticator, boolean issend)
	{
		byte[] buf = new byte[4 + 4 + 48]; //  4 for SMI code 
		// 4 for vendor type, vendor lengrth and salt
		// 48 for padded mod 16 32 + 1 key field
		byte[] keybuf = new byte[48];

		if (Log.DEBUG) Log.debug("Adding " + (issend ? " send " : " recv ") + "key to radius: " + Utils.hex(key));
		
		int mssmi = 311;
		// ms-smi code
		buf[0] = 0;
		buf[1] = 0;
		buf[2] = (311 & 0xFF00) >> 8;
		buf[3] = 311 & 0xff;

		buf[4] = (issend) ? RadiusPacket.MS_MPPE_SEND_KEY : RadiusPacket.MS_MPPE_RECV_KEY;
		buf[5] = 52; // key field + vendor attrib header

		// salt
		int salt = (int) (System.currentTimeMillis() & 0xFFFF);
		salt = salt | 0x80000000;  // spec
		
		buf[6] = (byte) (0xff & (salt >> 8));
		buf[7] = (byte) (0xff & salt);

		// Encrypt key
		keybuf[0] = 32; // length of unencrypted key in key
		System.arraycopy(key, 0, keybuf, 1, 32);

		for (int i = 33; i < 48; ++i)
			keybuf[i] = 0;

		// Now encrypt according to RFC 2548 2.4.2, 2.4.3

		byte[] bi = new byte[16];
		byte[] ci = new byte[16];
		byte[] md5in0 = new byte[secret.length + requestauthenticator.length + 2];
		byte[] md5in1 = new byte[secret.length + 16];
		byte[] md5in;
		
		// First secret.kenth of both are secret
		System.arraycopy(secret, 0, md5in1, 0, secret.length);
		System.arraycopy(secret, 0, md5in0, 0, secret.length);

		System.arraycopy(
			requestauthenticator,
			0,
			md5in0,
			secret.length,
			requestauthenticator.length);
			
		md5in0[secret.length + requestauthenticator.length] = (byte)(0xff & (salt >> 8));
		md5in0[secret.length + requestauthenticator.length + 1] = (byte)(0xff & salt);	
		for (int i = 0; i < 3; ++i)
		{
			if (0 == i)
			{
				md5in = md5in0;
			}
			else
			{
				md5in = md5in1;
				System.arraycopy(ci, 0, md5in, secret.length, 16);
			}
			new OpenSSL().md5(md5in, md5in.length, bi);
			for (int j = 0; j < 16; ++j)
			{
				ci[j] = (byte) (keybuf[(16 * i) + j] ^ bi[j]);
				buf[8 + (16 * i) + j] = ci[j];
			}
		}
		addAttribute(RadiusPacket.RADIUS_ATTR_VENDOR_SPECIFIC, buf);
	}

	/**
	 * @param eapstate
	 * @param secret
	 * @param requestauthenticator
	 */
	public void addMppeKeys(EAPState eapstate, byte[] secret, byte[] requestauthenticator)
	{
		addMppeKey(eapstate.getMpperecvkey(), secret, requestauthenticator, false);
		addMppeKey(eapstate.getMppesendkey(), secret, requestauthenticator, true);
	}

	public Object getAssociatedpacket()
	{
		return associatedpacket;
	}

	public void setAssociatedpacket(Object associatedpacket)
	{
		this.associatedpacket = associatedpacket;
	}

}
