/*
 * Created on Apr 17, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.ssl.SSL;
import com.instant802.cldc.ssl.SSLSessionContext;
import com.instant802.cldc.deviceconfiguration.ConfigurationModel;
import com.instant802.cldc.deviceconfiguration.WirelessUser;
import com.instant802.cldc.net.Datagram;
import com.instant802.cldc.util.Log;


/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class RadiusServer
{
	private Vector handlers;
	private Proxy proxy;
	private Hashtable wirelessusers;
	private String cert = "/etc/cacert.pem";
	private String key = "/etc/key.pem";
	private SSLSessionContext ctx = null;
	private EAPSessionTable eapsessionstate;
	private String TYPE = "RadiusServer";
	private final ConfigurationModel configModel;

	final int SSL_CONTEXT_MODE_PEER = 3;

	// Technically this should be per NAS - but since the Server is embedded with the NAS we cheat
	// blame Robin if you don't like this ...
	private byte[] sharedsecret = "secret".getBytes();

	public static final int ACCESS_REQUEST = 1,
		ACCESS_ACCEPT = 2,
		ACCESS_REJECT = 3,
		ACCESS_CHALLENGE = 11;

	public RadiusServer(Proxy proxy, ConfigurationModel configModel)
	{
		this.proxy = proxy;
		this.configModel = configModel;
		handlers = new Vector();
		wirelessusers = null;

		eapsessionstate = new EAPSessionTable();
		
		// Add handlers
		addHandler(new EAPHandler(this));

		// Set up SSL context
		ctx = new SSLSessionContext(SSL_CONTEXT_MODE_PEER);
		if (1 != ctx.useCertificateFile(cert, SSL.SSL_FILETYPE_PEM))
		{
			warning("Cannot use certificate:" + cert);
			ctx.printErrors();
			ctx.close();
			System.exit(-1);
		}
		if (1 != ctx.useRSAPrivateKey(key, SSL.SSL_FILETYPE_PEM))
		{
			warning("Cannot use key:" + cert);
			ctx.printErrors();
			ctx.close();
			System.exit(-1);
		}
	}

	public Proxy getProxy() {
		return this.proxy;
	}
	
	void setupInitialUsers() {
		//debug("starting - about to set up configuration from model "+configModel );
		if (null != configModel)
		{
			configModel.setRadiusServer(this);
			//debug("calling handleconfigchanged with model ");
			handleConfigChanged(configModel);
		}
	}

	public Hashtable getWIrelessUsers() {
		return wirelessusers;
	}
	
	private void warning(String string) {
		Log.warning( TYPE, string );
	}

	public EAPSessionTable getEAPSessionState()
	{
		return eapsessionstate;
	}

	public SSLSessionContext getSSLSessionContext()
	{
		return ctx;
	}

	public void addHandler(RadiusRequestHandler handler)
	{
		handlers.addElement(handler);
	}

	/**
	 * Process radius request packet rp by chain of Requesthandlers - stop on first succesful one.
	 * (Note that success means the handler could handle trhe packet -- not that the request succeeded).
	 * Also note that since each handler is responsible for generating responses, 
	 * lack of a matching handler
	 * will mean no response is sent which is a rejection, which is OK.
	 * @param rp
	 */
	public void processRequest(RadiusPacket rp)
	{
		int hsize = handlers.size();
		boolean handled = false;
		//debug("Processing Radius Request");
		try
		{
			for (int i = 0; i < hsize; ++i)
			{
				RadiusRequestHandler handler = (RadiusRequestHandler) handlers.elementAt(i);
				if (handler.handled(rp, sharedsecret))
				{
					handled = true;
					break;
				}
			}
		}
		catch (Exception e)
		{
			warning( e.toString() );
			return;

		}
		if (!handled)
			warning("Couldn't find handler for packet ");
	}

	/**
	 * @param respbuf
	 */
	public void respond(byte[] respbuf)
	{
		try
		{
			Datagram p =new Datagram(respbuf);
			proxy.getForwarder().transactionEnd(new RadiusPacket(p));
		}
		catch (Exception e)
		{
			error( new Exception( e.toString()));
		}
	}

	private WirelessUser getUser(byte[] user) {
		WirelessUser wu;

		synchronized (this)
		{
			wu = (WirelessUser) wirelessusers.get(new String(user));
		}
		if (null != wu && ! wu.disabled())
			return wu;
		return null;
	}
	
	public byte[] getPlaintextPassword(byte[] user)
	{
		String pwd;
		WirelessUser wu = getUser(user);
		
		if (null != (pwd =  wu.plaintextPassword()))
				return pwd.getBytes();
		return null;
	}
	
	public byte[] getNTHashedPassword(byte[] user)
	{
		String pwd;
		WirelessUser wu = getUser(user);
		byte[] hash = new byte[16], pwarray;
		byte val;
		
		if (null != (pwd =  wu.hashedPassword())) {
			pwarray = pwd.getBytes();
			for (int i = 0; i < 16; ++i) {
				val = (byte)(pwarray[1 + (2 * i)] - 'A');
				val = (byte) ((val << 4) + (byte)(pwarray[2 * i] - 'A'));
				hash[i] = val;
			}
			return hash;
		}
		return null;
	}

	/**
	 * Some protocols (e.g. Chap) need a 'name'. This is it.
	 */
	public byte[] getSystemName()
	{
		return "rootin-tootin-ap-radius".getBytes();
	}

	/* 
	 *  Called by Access Point config when conifg changes
	 */
	 
	public synchronized void handleConfigChanged(ConfigurationModel model)
	{
		if (null != model)
		{
			try
			{ // bugs ?? - temp measure
				/* passwords is a hashtable which contains 'WirelessUser' objects
				 * which are indexed by the username.  The radius server first attempts
				 * to get these from the 
				 */
				wirelessusers =  model.getWirelessUsers();
			} catch (Exception e) {
				error( e );
				wirelessusers = null;
			}
		}
	}

	private void error(Exception e) {
		Log.error( TYPE, e );
	}
}
