/*
 * Created on Oct 1, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.synchronization;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.ConfigurationChangeListener;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.LSQL.LSQLResultSet;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.deviceconfiguration.ConfigObject;
import com.instant802.cldc.deviceconfiguration.ConfigurationModel;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.deviceconfiguration.ObjectList;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class SynchronizationAgent implements ConfigurationChangeListener {

	private final OrchestratorNode node;

	public SynchronizationAgent(OrchestratorNode node) {
		this.node = node;
	}

	public void handleSynchronizationRequest(RequestSynchronizationMessage message)
		throws ElementNotFoundException, IOException, SocketException {
		if (Log.DEBUG) info("received synchronization request");
		message.sendReply(
			node,
			node.configurationModel().asSynchronizationSet());
		if (Log.DEBUG) info("sent synchronization reply");
	}

	public void synchronizeCluster() throws LSQLException, APConfException {
		 node.configurationModel().handleConfigurationChange();
	}
	
	public void synchronize(SynchronizationSetMessage message)
		throws
			XMLRepresentationException,
			IOException,
			MalformedHTTPMessageException,
			LSQLException,
			APConfException, ElementNotFoundException, InvalidTimingException, SocketException, AgentNotRunningException {
		if (Log.DEBUG) Log.notice("Processing configuration update from "+message.getSenderIp() );
		ObjectList objects = message.currentObjects();
		if (Log.DEBUG) info( "read objects from message");
		ConfigurationModel model = node.configurationModel();
		if (Log.DEBUG) info( "got configuration model from node" );
		if (Log.DEBUG) debug("incoming configuration", objects );
		if (Log.DEBUG) debug("current configuration", model );
		debugWait();
		synchronized (model) {
			debugAcquire();
			boolean locallyModifiedSet = true;
			try {
				if (Log.DEBUG) info("updating from objects");
				model.updateFromObjectList(objects);
				if (Log.DEBUG) info("writing to apconf");
				model.writeToAPConf();
				if (Log.DEBUG) info("clearing locally modified flag");
				node.lsqlMessageReceiver().clearLocallyModified();
				locallyModifiedSet = false;
				if (Log.DEBUG) info("notifying radius server");
				model.notifyRadiusServer();
				if (Log.DEBUG) info("informing membership agent");
				node.membershipAgent().configurationSynchronized();
				if (Log.DEBUG) info("checking for deletions");
				node.coordinationAgent().checkForDeletions();
			} finally {
				debugRelease();
				if( locallyModifiedSet ) node.lsqlMessageReceiver().clearLocallyModified();
			}
		}
		// here's the patch for pushing location out if it got changed asynchronously
		if( model.patch() ) {
			model.distributeConfiguration();
		}
		if (Log.DEBUG) Log.info("done with synchronization");
	}

	private void debug(String name, ObjectList list) {
		if (Log.DEBUG) Log.debug("***** "+name+" configuration size:"+list.size() );
		list.debugDump();
		if (Log.DEBUG) Log.debug("***** end "+name+" configuration");
	}

	
	
	private void debugRelease() {
		if (Log.DEBUG) info("releasing lock on configuration model");
	}

	private void debugAcquire() {
		if (Log.DEBUG) info("acquired lock on configuration model");
	}

	private void debugWait() {
		if (Log.DEBUG) info("waiting for lock on configuration model.");
	}

	private void info(String string) {
		Log.info( string );
	}

	public void handleConfigurationChange() throws XMLRepresentationException, IOException, 
		MalformedHTTPMessageException, LSQLException, APConfException 
	{
		node.lsqlMessageReceiver().setLocallyModifiedIfClustered();
	}

	public void handleObjectChange(ConfigObject object) {
		node.objectChangeProcessor().handleObjectChange( object );
	}
}
