/*
 * Created on Aug 21, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.util;

import java.util.Vector;


/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ControlledQueue {
	private boolean closed = false;
	private Vector list = new Vector();

	/**
	 * add an object to the queue, to be dequeued after all current entries.
	 * @param o the object to add
	 * @throws QueueClosedException thrown if entries are removed
	 */
	public synchronized void enqueue(Object o) throws QueueClosedException
	{
		checkOpen();
		list.addElement(o);
		notifyAll();
	}

	/**
	 * return an object from the queue if there is one available, otherwise block until an object
	 * becomes available.  When the queue is empty and the queue is closed, this method
	 * will throw an exception to indicate that no further elements can ever be retrieved from
	 * this queue. 
	 * @return an element.
	 * @throws QueueClosedException
	 */
	public synchronized Object dequeue() throws QueueClosedException
	{
		while (list.size() <= 0)
		{
			checkOpen();
			try {
				wait();
			} catch (InterruptedException e) {
				/* if the queue has been closed, this will be discovered shortly */
			}
		}
		Object retrieved = list.elementAt(0);
		list.removeElementAt(0);
		list.trimToSize();
		return retrieved;
	}

	private void checkOpen() throws QueueClosedException {
		if( closed ) {
			throw new QueueClosedException();
		}
	}

	public synchronized boolean isOpen() {
		return ! closed;
	}

	/* close the queue - this prevents new elements from being added, and 
	 * causes attempts to dequeue to throw an exception when the queue has been
	 * drained of entries.
	 */
	public synchronized void close() {
		closed = true;
		notifyAll();	
	}
}
