/*
 * Created on Jun 9, 2004
 * author: Robin Barooah
 * Copyright (C) 2004 Instant802 Networks, Inc. All Rights Reserved.
 */
package com.instant802.cldc.util;

/**
 * Simple parser for lines of fields separated by tab characters.
 * Does not support any form of quoting, so tabs are always treated
 * as field delimiters.
 * @author robin
 */
public class SimpleDelimParser {

	private final String tsv;
	private int start = 0, pos = 0;
	private char delim;
	
	/**
	 * create a parser to parse the supplied string.
	 */
	public SimpleDelimParser( String toParse, char delim ) {
		this.tsv = toParse;
		this.delim = delim;
	}
	
	/**
	 * If more fields are avaiable, return the next one, otherwise return null.
	 * a tab at the end of the line is ignored.
	 */
	public String nextField() {		
		
		pos = start;
		
		// we are already at the end of the line
		if( start >= tsv.length() ) {
			return null;
		}
		
		// scan for the next delim or the end of the line
		while( tsv.charAt( pos ) != delim ) {
			pos ++;
			if( pos >= tsv.length() ) {
				break;
			}
		}
				
		// return between start and the tab we found
		String found = tsv.substring( start, pos );
		
		// next time, start from just after the tab
		start = pos + 1; 	
		return found;
	}
}
